@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 /*
-$Id: extprog_toolkit.c,v 2.2 2003/05/05 16:06:58 ericprev Exp $
+$Id: extprog_toolkit.c,v 2.3 2003/08/21 17:33:38 ericprev Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -241,16 +241,19 @@ void kick_user(int sck, const char *nick, const char *kicknick, const char *reas
 /****************************************************/
 /* input: sck: socket to send the DC string         */
 /*        nick: your nickname                       */
+/*        duration: bantime default minutes         */
+/*                  accepts parameter h -> hours    */
+/*                  or d -> days                    */
 /*        kicknick: nickname of the user to ban     */
 /*        reason: a short message giving the reason */
 /*                NOTE: the message cannot contains */
 /*                " characters                      */
 /****************************************************/
-void ban_user(int sck, const char *nick, const char *kicknick, const char *reason)
+void ban_user(int sck, const char *nick, const char *kicknick, const char *duration, const char *reason)
 {
 	GString *output;
 	output=g_string_sized_new(512);
-	g_string_sprintf(output,"$To: Hub-Security From: %s $<%s> -pbankick %s \"%s\"",nick,nick,kicknick,reason);
+	g_string_sprintf(output,"$To: Hub-Security From: %s $<%s> -ban %s %s \"%s\"",nick,nick,duration,kicknick,reason);
 
 	send_dc_line(sck,output->str,NULL);
 	g_string_free(output,TRUE);
