@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 /*
-$Id: global_user.c,v 2.13 2003/02/26 08:20:45 ericprev Exp $
+$Id: global_user.c,v 2.19 2003/11/05 18:16:18 ericprev Exp $
 */
 
 
@@ -159,8 +159,8 @@ void glus_free_user_info_ptrarray(GPtrArray *glus_info_ptr_array)
 /* output: 0=ok else user not found                                                   */
 /* NOTE: the GString reference is stolen, you must not free it.                       */
 /**************************************************************************************/
-void glus_send_to_named_user(const char *nickname, unsigned int uflag_msk, unsigned int wanted_uflag,
-                             unsigned int eflag_msk, unsigned int wanted_eflag, GString *msg)
+void glus_send_to_named_user(const char *nickname, guint16 uflag_msk, guint16 wanted_uflag,
+                             guint16 eflag_msk, guint16 wanted_eflag, GString *msg)
 {
 	GLUS_PARAM nw;
 	int i;
@@ -272,7 +272,8 @@ void glus_hub_users_stat(guint64 *shared_size, unsigned int *nb_users)
 	{
 		for(i=0;i<glus_array->len;i++)
 		{
-			(((GLOBAL_USER_CALLBACKS*)(g_ptr_array_index(glus_array,i)))->add_hub_users_stat)(shared_size,nb_users);
+			if((((GLOBAL_USER_CALLBACKS*)(g_ptr_array_index(glus_array,i)))->add_hub_users_stat)!=NULL)
+				(((GLOBAL_USER_CALLBACKS*)(g_ptr_array_index(glus_array,i)))->add_hub_users_stat)(shared_size,nb_users);
 		}
 	}
 }
@@ -301,6 +302,70 @@ void glus_kick_named_user(const char *kicking_nickname,const char *nickname)
 	}
 }
 
+/******************************************/
+/* ban the user having the given nickname */
+/******************************************/
+void glus_ban_named_user(const char *kicking_nickname,const char *nickname, unsigned long duration)
+{
+	int i;
+	int ret;
+
+	if(glus_array)
+	{
+		for(i=0;i<glus_array->len;i++)
+		{
+			if(((GLOBAL_USER_CALLBACKS*)(g_ptr_array_index(glus_array,i)))->ban_named_user!=NULL)
+			{
+				ret=(((GLOBAL_USER_CALLBACKS*)(g_ptr_array_index(glus_array,i)))->ban_named_user)(kicking_nickname,nickname, duration);
+				if(ret==0)		/* user found */
+					break;
+			}
+		}
+	}
+}
+
+/************************************/
+/* ban the user having the given ip */
+/************************************/
+void glus_ban_ip(const char *kicking_nickname, const struct in_addr ip, unsigned long duration)
+{
+	int i;
+	int ret;
+
+	if(glus_array)
+	{
+		for(i=0;i<glus_array->len;i++)
+		{
+			if(((GLOBAL_USER_CALLBACKS*)(g_ptr_array_index(glus_array,i)))->ban_ip!=NULL)
+			{
+				ret=(((GLOBAL_USER_CALLBACKS*)(g_ptr_array_index(glus_array,i)))->ban_ip)(kicking_nickname, ip, duration);
+				if(ret==0)		/* user found */
+					break;
+			}
+		}
+	}
+}
+
+/**************************************************/
+/* unban the user having the given nickname or IP */
+/**************************************************/
+void glus_uban_named_user(const char *nickname)
+{
+	int i;
+
+	if(glus_array)
+	{
+		for(i=0;i<glus_array->len;i++)
+		{
+			if(((GLOBAL_USER_CALLBACKS*)(g_ptr_array_index(glus_array,i)))->uban_named_user!=NULL)
+			{
+				/* we relay the unban to all entries */
+				(((GLOBAL_USER_CALLBACKS*)(g_ptr_array_index(glus_array,i)))->uban_named_user)(nickname);
+			}
+		}
+	}
+}
+
 /*************************************************/
 /* disconnect the user having the given nickname */
 /*************************************************/
@@ -320,9 +385,9 @@ void glus_disconnect_named_user(const char *nickname)
 	}
 }
 
-/******************************************************/
-/* notify everyone a user is disconneted from the hub */
-/******************************************************/
+/*******************************************************/
+/* notify everyone a user is disconnected from the hub */
+/*******************************************************/
 void glus_user_is_disconnected(const char *nickname)
 {
 	int i;
@@ -378,7 +443,7 @@ void glus_do_quit(const char *nickname)
 /******************************************************/
 /* function used to say a "$MyINFO" on the connection */
 /******************************************************/
-void glus_do_my_info(const char *nickname, const char *description, const char *user_cnx_type, const unsigned int user_flag, const char *user_mail, const guint64 shared_size, const unsigned int ext_flag, const unsigned int privilege, const char *client_version)
+void glus_do_my_info(const char *nickname, const char *description, const char *user_cnx_type, const guint16 user_flag, const char *user_mail, const guint64 shared_size, const guint16 ext_flag, const gint16 level, const char *client_version)
 {
 	int i;
 
@@ -387,7 +452,7 @@ void glus_do_my_info(const char *nickname, const char *description, const char *
 		for(i=0;i<glus_array->len;i++)
 		{
 			if((((GLOBAL_USER_CALLBACKS*)(g_ptr_array_index(glus_array,i)))->user_do_my_info)!=NULL)
-				(((GLOBAL_USER_CALLBACKS*)(g_ptr_array_index(glus_array,i)))->user_do_my_info)(nickname,description,user_cnx_type,user_flag,user_mail,shared_size,ext_flag,privilege, client_version);
+				(((GLOBAL_USER_CALLBACKS*)(g_ptr_array_index(glus_array,i)))->user_do_my_info)(nickname,description,user_cnx_type,user_flag,user_mail,shared_size,ext_flag,level, client_version);
 		}
 	}
 }
@@ -458,8 +523,7 @@ GString *txt_glus_get_user_info(const char *nick)
 					((gui->user_mail->str!=NULL)?gui->user_mail->str:""),
 					((gui->user_description->str!=NULL)?gui->user_description->str:""),
 					gui->user_flag,
-					gui->privilege
-					);
+					gui->level);
 #else
 		g_string_sprintf(str,"%s$%Lu$%s$%s$%u$%d$",
 					((gui->user_cnx_type!=NULL)?gui->user_cnx_type:""),
@@ -467,8 +531,7 @@ GString *txt_glus_get_user_info(const char *nick)
 					((gui->user_mail->str!=NULL)?gui->user_mail->str:""),
 					((gui->user_description->str!=NULL)?gui->user_description->str:""),
 					gui->user_flag,
-					gui->privilege
-					);
+					gui->level);
 #endif
 		glus_free_user_info(gui);
 	}
