@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 /*
-$Id: hub_cnx_handshake.c,v 2.12 2003/04/13 07:50:18 ericprev Exp $
+$Id: hub_cnx_handshake.c,v 2.13 2003/10/19 12:48:10 blusseau Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -42,6 +42,7 @@ $Id: hub_cnx_handshake.c,v 2.12 2003/04/13 07:50:18 ericprev Exp $
 #include "hub_passwd.h"
 #include "md5.h"
 #include "network.h"
+#include "db_xml.h"
 
 static GArray *hub_in_handshake=NULL;
 
@@ -548,14 +549,12 @@ GED_CALLBACKS *hub_handshake_init(void)
 	char *hubname;
 
 	/* retrieve the HUBNAME (if not defined, it is redefined to "noname") */
-	G_LOCK(conf_file);
-	hubname=e_db_str_get(conf_file,"HUBNAME");
+	hubname=db_str_get("HUBNAME");
 	if(hubname==NULL)
 	{
-		e_db_str_set(conf_file,"HUBNAME","noname");
+		db_str_set("HUBNAME","noname");
 		hubname=strdup("noname");
 	}
-	G_UNLOCK(conf_file);
 	if(!strcmp(hubname,"noname"))
 	{
 		fprintf(stderr,"WARNING: you must use a different HUBNAME value. Use -dbset to modify it\n"
@@ -578,9 +577,7 @@ GED_CALLBACKS *hub_handshake_init(void)
 		}
 		hub_id[MD_BLOC_SIZE*2]='\0';
 
-		G_LOCK(conf_file);
-		e_db_str_set(conf_file,"HUB_ID",hub_id);
-		G_UNLOCK(conf_file);
+		db_str_set("HUB_ID",hub_id);
 	}
 
 	hub_in_handshake=g_array_new(FALSE,FALSE,sizeof(HUB_HSHAKE));
