@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 /*
-$Id: hub_cnx_lst.c,v 2.23 2003/05/06 16:44:11 ericprev Exp $
+$Id: hub_cnx_lst.c,v 2.34 2003/11/22 13:47:08 blusseau Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -35,7 +35,14 @@ $Id: hub_cnx_lst.c,v 2.23 2003/05/06 16:44:11 ericprev Exp $
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#ifdef WITH_GCRYPT
+#include <gcrypt.h>
+#define MD4_DIGEST_LENGTH 16
+#define MD4_ALGO(source,len,dest)   gcry_md_hash_buffer(GCRY_MD_MD4,dest,source,len)
+#else
 #include <openssl/md4.h>
+#define MD4_ALGO MD4
+#endif
 #include <glib.h>
 
 #include "gvar.h"
@@ -46,6 +53,8 @@ $Id: hub_cnx_lst.c,v 2.23 2003/05/06 16:44:11 ericprev Exp $
 #include "toolkit.h"
 #include "md5.h"
 #include "md_db.h"
+#include "users_xml.h"
+#include "db_xml.h"
 
 /* list of clusters */
 /* the value is a CLUSTER_ENTRY * */
@@ -226,7 +235,7 @@ static HUB_USER_ENTRY *find_user_named_for_this_hub(HUB_CNX_ENTRY *hce, const un
 	hue->user_description=g_string_new("");
 	hue->user_flag=1;		/* LSB always set */
 	hue->ext_flag=0;
-	hue->privilege=0;
+	hue->level=0;
 	hue->client_version=g_string_new("");
 
   	g_hash_table_insert(hce->hub_users,hue->user_nick->str,hue);
@@ -293,7 +302,7 @@ static void end_user_connection(LOCAL_USER_CNX_ENTRY *luce, gboolean with_remove
 /* NOTE: to cluster, we resend the chunk as is (on add or update, not if unchanged */
 /***********************************************************************************/
 /* Require: SC_USER_NICK. Optional: SC_USER_SHARE,SC_USER_MAIL,SC_USER_DESC */
-/* SC_USER_FLAG,SC_USER_EFLAG,SC_USER_PRIV,SC_USER_CNX_TYPE,SC_USER_CLIVER  */
+/* SC_USER_FLAG,SC_USER_EFLAG,SC_USER_LEVEL,SC_USER_CNX_TYPE,SC_USER_CLIVER */
 /****************************************************************************/
 /* don't free the CHUNK_CORE */
 /*****************************/
@@ -318,7 +327,7 @@ static void hub_cc_user_add_or_update(HUB_CNX_ENTRY *hce,CHUNK_CORE *cc)
 	CHUNK_GET_AND_SET_NNSTR_in_gstring_ND(hue->user_description,cc,SC_USER_DESC,is_updated);
 	CHUNK_GET_AND_SET_UVAL_ND(hue->user_flag,cc,SC_USER_FLAG,is_updated);
 	CHUNK_GET_AND_SET_UVAL_ND(hue->ext_flag,cc,SC_USER_EFLAG,is_updated);
-	CHUNK_GET_AND_SET_UVAL_ND(hue->privilege,cc,SC_USER_PRIV,is_updated);
+	CHUNK_GET_AND_SET_SVAL_ND(hue->level,cc,SC_USER_LEVEL,is_updated);
 
 	/* USER_CNX_TYPE uses a static string, not an allocated one */
 	sc=chunk_get_subcore(cc,SC_USER_CNX_TYPE);
@@ -352,7 +361,7 @@ static void hub_cc_user_add_or_update(HUB_CNX_ENTRY *hce,CHUNK_CORE *cc)
 		excluded_hub_entry=g_list_prepend(excluded_hub_entry,hce);
 		glus_do_my_info(hue->user_nick->str,hue->user_description->str, hue->user_cnx_type,
 		                hue->user_flag,hue->user_mail->str,hue->shared_size,
-		                hue->ext_flag,hue->privilege,hue->client_version->str);
+		                hue->ext_flag,hue->level,hue->client_version->str);
 		excluded_hub_entry=g_list_remove(excluded_hub_entry,hce);
 	}
 }
@@ -402,7 +411,7 @@ static void add_user_to_user_lst_for_each_hub(GLUS_USER_INFO *gui, CHUNK_CORE *c
 	guint64 share=GUINT64_TO_BE(gui->shared_size);
 	guint16 uflag=GUINT16_TO_BE(gui->user_flag);
 	guint16 eflag=GUINT16_TO_BE(gui->ext_flag);
-	guint16 priv=GUINT16_TO_BE(gui->privilege);
+	gint16 lvl=GINT16_TO_BE(gui->level);
 
 	chunk_core_append_subchunk(cc,SC_USER_NICK,gui->user_nick->len,gui->user_nick->str);
 	chunk_core_append_subchunk(cc,SC_USER_SHARE,sizeof(share),&share);
@@ -410,7 +419,7 @@ static void add_user_to_user_lst_for_each_hub(GLUS_USER_INFO *gui, CHUNK_CORE *c
 	chunk_core_append_subchunk(cc,SC_USER_DESC,gui->user_description->len,gui->user_description->str);
 	chunk_core_append_subchunk(cc,SC_USER_FLAG,sizeof(uflag),&uflag);
 	chunk_core_append_subchunk(cc,SC_USER_EFLAG,sizeof(eflag),&eflag);
-	chunk_core_append_subchunk(cc,SC_USER_PRIV,sizeof(priv),&priv);
+	chunk_core_append_subchunk(cc,SC_USER_LEVEL,sizeof(lvl),&lvl);
 	chunk_core_append_subchunk(cc,SC_USER_CNX_TYPE,strlen(gui->user_cnx_type),gui->user_cnx_type);
 	chunk_core_append_subchunk(cc,SC_USER_CLIVER,gui->client_version->len,gui->client_version->str);
 }
@@ -470,6 +479,7 @@ static void hub_cc_user_lst(HUB_CNX_ENTRY *hce,CHUNK_CORE *cc)
 	int sc_num=0;
 	guint64 __old_val_u64;
 	guint16 __old_val_u16;
+	gint16 __old_val_s16;
 
 	excluded_hub_entry=g_list_prepend(excluded_hub_entry,hce);
 
@@ -496,7 +506,7 @@ static void hub_cc_user_lst(HUB_CNX_ENTRY *hce,CHUNK_CORE *cc)
 												/* to avoid loop, we don't send a message on the link where it came from */
 												glus_do_my_info(hue->user_nick->str,hue->user_description->str, hue->user_cnx_type,
 		                										hue->user_flag,hue->user_mail->str,hue->shared_size,
-		                										hue->ext_flag,hue->privilege,hue->client_version->str);
+		                										hue->ext_flag,hue->level,hue->client_version->str);
 											}
 											hue=NULL;
 										}
@@ -539,10 +549,10 @@ static void hub_cc_user_lst(HUB_CNX_ENTRY *hce,CHUNK_CORE *cc)
 											is_updated=TRUE;
 										break;
 
-			case SC_USER_PRIV:
-										__old_val_u16=hue->privilege;
-										hue->privilege=subchunk_get_guint(sc);
-										if(hue->privilege!=__old_val_u16)
+			case SC_USER_LEVEL:
+										__old_val_s16=hue->level;
+										hue->level=subchunk_get_gint(sc);
+										if(hue->level!=__old_val_s16)
 											is_updated=TRUE;
 										break;
 
@@ -586,7 +596,7 @@ static void hub_cc_user_lst(HUB_CNX_ENTRY *hce,CHUNK_CORE *cc)
 			/* to avoid loop, we don't send a message on the link where it came from */
 			glus_do_my_info(hue->user_nick->str,hue->user_description->str, hue->user_cnx_type,
          										hue->user_flag,hue->user_mail->str,hue->shared_size,
-         										hue->ext_flag,hue->privilege,hue->client_version->str);
+         										hue->ext_flag,hue->level,hue->client_version->str);
 		}
 	}
 
@@ -621,7 +631,7 @@ static void hub_cc_user_info_req(HUB_CNX_ENTRY *hce,CHUNK_CORE *cc)
 		guint64 share=GUINT64_TO_BE(gui->shared_size);
 		guint16 uflag=GUINT16_TO_BE(gui->user_flag);
 		guint16 eflag=GUINT16_TO_BE(gui->ext_flag);
-		guint16 priv=GUINT16_TO_BE(gui->privilege);
+		gint16 lvl=GINT16_TO_BE(gui->level);
 
 		gba=chunk_convert_param_to_mem(CC_USER_IN,9,
 													SC_PARAM_PTR(SC_USER_NICK,gui->user_nick->len,gui->user_nick->str),
@@ -630,7 +640,7 @@ static void hub_cc_user_info_req(HUB_CNX_ENTRY *hce,CHUNK_CORE *cc)
 													SC_PARAM_PTR(SC_USER_DESC,gui->user_description->len,gui->user_description->str),
 													SC_PARAM_GUINT16(SC_USER_FLAG,uflag),
 													SC_PARAM_GUINT16(SC_USER_EFLAG,eflag),
-													SC_PARAM_GUINT16(SC_USER_PRIV,priv),
+													SC_PARAM_GINT16(SC_USER_LEVEL,lvl),
 													SC_PARAM_PTR(SC_USER_CNX_TYPE,strlen(gui->user_cnx_type),gui->user_cnx_type),
 													SC_PARAM_PTR(SC_USER_CLIVER,gui->client_version->len,gui->client_version->str)
 												);
@@ -743,6 +753,134 @@ static void hub_cc_user_kick(HUB_CNX_ENTRY *hce,CHUNK_CORE *cc)
 		g_string_free(kicker_nick,TRUE);
 }
 
+/***************************************************************************/
+/* ban the user in the chunk. The change must be relay to everyone but hce */
+/***************************************************************************/
+/* NOTE: to cluster, we resend the chunk as is */
+/***********************************************/
+/* Require: SC_USER_NICK, SC_KICK_DURATION */
+/*****************************/
+/* don't free the CHUNK_CORE */
+/*****************************/
+static void hub_cc_user_ban(HUB_CNX_ENTRY *hce,CHUNK_CORE *cc)
+{
+	SUBCHUNK_CORE *sc;
+	GString *kicked_nick;
+	GString *kicker_nick;
+	unsigned long duration=24*3600;	/* default duration is 24 hours */
+	gboolean have_duration=FALSE;
+
+	sc=chunk_get_subcore(cc,SC_USER_NICK);
+	if(sc==NULL)
+		return;	/* invalid CC_USER_IN chunk */
+
+	kicked_nick=g_string_new("");
+	subchunk_assign_gstring(kicked_nick,sc);
+	
+	sc=chunk_get_subcore(cc,SC_KICK_KICKER);
+	if(sc==NULL)
+		kicker_nick=NULL;
+	else
+	{
+		kicker_nick=g_string_new("");
+		subchunk_assign_gstring(kicker_nick,sc);
+	}
+
+	CHUNK_GET_AND_SET_UVAL_ND(duration,cc,SC_KICK_DURATION,have_duration)
+	
+	/* the glus_kick function does everything, we just add hce to the excluded hub list to prevent the message to go back */
+	excluded_hub_entry=g_list_prepend(excluded_hub_entry,hce);
+	glus_ban_named_user((kicker_nick!=NULL)?kicker_nick->str:NULL,
+	                     kicked_nick->str, duration);
+	excluded_hub_entry=g_list_remove(excluded_hub_entry,hce);
+
+	/* free memory */
+	g_string_free(kicked_nick,TRUE);
+	if(kicker_nick)
+		g_string_free(kicker_nick,TRUE);
+}
+
+/*************************************************************************/
+/* ban the IP in the chunk. The change must be relay to everyone but hce */
+/*************************************************************************/
+/* NOTE: to cluster, we resend the chunk as is */
+/***********************************************/
+/* Require: SC_USER_IP, SC_KICK_DURATION */
+/*****************************************/
+/* don't free the CHUNK_CORE */
+/*****************************/
+static void hub_cc_user_banip(HUB_CNX_ENTRY *hce,CHUNK_CORE *cc)
+{
+	SUBCHUNK_CORE *sc;
+	GString *kicker_nick;
+	unsigned long duration=24*3600;	/* default duration is 24 hours */
+	struct in_addr ip;
+	gboolean have_duration=FALSE;
+	gboolean have_ip=FALSE;
+
+	sc=chunk_get_subcore(cc,SC_USER_IP);
+	if(sc==NULL)
+		return;	/* invalid CC_USER_IN chunk */
+
+	CHUNK_GET_AND_SET_UVAL_ND(ip.s_addr,cc,SC_USER_IP,have_ip);
+	if(have_ip==FALSE)
+	{
+		fprintf(stderr,"hub<=>hub: error, hub_cc_user_banip: no SC_USER_IP chunk\n");
+		return;
+	}
+	
+	sc=chunk_get_subcore(cc,SC_KICK_KICKER);
+	if(sc==NULL)
+		kicker_nick=NULL;
+	else
+	{
+		kicker_nick=g_string_new("");
+		subchunk_assign_gstring(kicker_nick,sc);
+	}
+
+	CHUNK_GET_AND_SET_UVAL_ND(duration,cc,SC_KICK_DURATION,have_duration)
+	
+	/* the glus_ban_ip function does everything, we just add hce to the excluded hub list to prevent the message to go back */
+	excluded_hub_entry=g_list_prepend(excluded_hub_entry,hce);
+	glus_ban_ip((kicker_nick!=NULL)?kicker_nick->str:NULL,
+					ip,duration);
+	excluded_hub_entry=g_list_remove(excluded_hub_entry,hce);
+
+	/* free memory */
+	if(kicker_nick)
+		g_string_free(kicker_nick,TRUE);
+}
+
+/*****************************************************************************/
+/* unban the user in the chunk. The change must be relay to everyone but hce */
+/*****************************************************************************/
+/* NOTE: to cluster, we resend the chunk as is */
+/***********************************************/
+/* Require: SC_USER_NICK */
+/*****************************/
+/* don't free the CHUNK_CORE */
+/*****************************/
+static void hub_cc_user_uban(HUB_CNX_ENTRY *hce,CHUNK_CORE *cc)
+{
+	SUBCHUNK_CORE *sc;
+	GString *kicked_nick;
+
+	sc=chunk_get_subcore(cc,SC_USER_NICK);
+	if(sc==NULL)
+		return;	/* invalid CC_USER_IN chunk */
+
+	kicked_nick=g_string_new("");
+	subchunk_assign_gstring(kicked_nick,sc);
+	
+	/* the glus_kick function does everything, we just add hce to the excluded hub list to prevent the message to go back */
+	excluded_hub_entry=g_list_prepend(excluded_hub_entry,hce);
+	glus_uban_named_user(kicked_nick->str);
+	excluded_hub_entry=g_list_remove(excluded_hub_entry,hce);
+
+	/* free memory */
+	g_string_free(kicked_nick,TRUE);
+}
+
 /**********************************************************************************/
 /* disconnect the user in the chunk. The change must be relay to everyone but hce */
 /**********************************************************************************/
@@ -936,6 +1074,18 @@ static void process_hub_incoming_chunks(CLUSTER_ENTRY *ce,HUB_CNX_ENTRY *hce)
 							hub_cc_user_kick(hce,cc);
 							break;
 
+			case CC_USER_BAN:
+							hub_cc_user_ban(hce,cc);
+							break;
+
+			case CC_USER_BANIP:
+							hub_cc_user_banip(hce,cc);
+							break;
+
+			case CC_USER_UBAN:
+							hub_cc_user_uban(hce,cc);
+							break;
+
 			case CC_USER_DISC:
 							hub_cc_user_disc(hce,cc);
 							break;
@@ -1049,13 +1199,13 @@ static void hc_user_do_quit(const char *nickname)
 /******************************************************/
 /* function used to say a "$MyINFO" on the connection */
 /******************************************************/
-static void hc_user_do_myinfo(const char *nickname, const char *description, const char *user_cnx_type, const unsigned int user_flag, const char *user_mail, const guint64 shared_size, const unsigned int ext_flag, const unsigned int privilege, const char *client_version)
+static void hc_user_do_myinfo(const char *nickname, const char *description, const char *user_cnx_type, const guint16 user_flag, const char *user_mail, const guint64 shared_size, const guint16 ext_flag, const gint16 level, const char *client_version)
 {
 	GByteArray *gba;
 	guint64 share=GUINT64_TO_BE(shared_size);
 	guint16 uflag=GUINT16_TO_BE(user_flag);
 	guint16 eflag=GUINT16_TO_BE(ext_flag);
-	guint16 priv=GUINT16_TO_BE(privilege);
+	gint16 lvl=GINT16_TO_BE(level);
 
 	gba=chunk_convert_param_to_mem(CC_USER_IN,9,
 												SC_PARAM_PTR(SC_USER_NICK,strlen(nickname),nickname),
@@ -1064,7 +1214,7 @@ static void hc_user_do_myinfo(const char *nickname, const char *description, con
 												SC_PARAM_PTR(SC_USER_DESC,strlen(description),description),
 												SC_PARAM_GUINT16(SC_USER_FLAG,uflag),
 												SC_PARAM_GUINT16(SC_USER_EFLAG,eflag),
-												SC_PARAM_GUINT16(SC_USER_PRIV,priv),
+												SC_PARAM_GINT16(SC_USER_LEVEL,lvl),
 												SC_PARAM_PTR(SC_USER_CNX_TYPE,strlen(user_cnx_type),user_cnx_type),
 												SC_PARAM_PTR(SC_USER_CLIVER,strlen(client_version),client_version)
 											);
@@ -1110,7 +1260,7 @@ static void hc_send_to_all_users(GLUS_PARAM *glus_param)
 #warning user_flag and ext_flag not honored 
 
 	gba=chunk_convert_param_to_mem(CC_BCAST,1,
-													SC_PARAM_PTR(SC_MSG_CONTENT,glus_param->msg->data->len,glus_param->msg->data->str));
+								   SC_PARAM_PTR(SC_MSG_CONTENT,glus_param->msg->data->len,glus_param->msg->data->str));
 
 	send_chunk_to_all_hubs(gba);	/* don't free gba */
 }
@@ -1159,7 +1309,7 @@ static GLUS_USER_INFO *hc_glus_get_user_info_from_hue(HUB_USER_ENTRY *hue)
 		gui->user_mail=g_string_new("");
 	gui->user_flag=hue->user_flag;
 	gui->ext_flag=hue->ext_flag;
-	gui->privilege=hue->privilege;
+	gui->level=hue->level;
 	if(hue->client_version)
 		gui->client_version=g_string_new(hue->client_version->str);
 	else
@@ -1252,8 +1402,6 @@ void hc_glus_get_users_info(GPtrArray *array_of_gui_ptr)
 	HL_UNLOCK_READ(cluster_entry_lock);
 }
 
-
-/* -------------------------------------------------------------------------- */
 /************************************/
 /* add a user to user (and op) list */
 /*****************************************************************************/
@@ -1271,7 +1419,7 @@ static void hc_get_nick_lists_two(gpointer key, gpointer value, gpointer user_da
 	g_string_append(lists[0],hue->user_nick->str);
 	g_string_append(lists[0],"$$");
 
-	if(hue->privilege & HAVE_OP_KEY)
+	if(hue->level >= get_right_level(OPERATOR))
 	{
 		g_string_append(lists[1],hue->user_nick->str);
 		g_string_append(lists[1],"$$");
@@ -1333,7 +1481,6 @@ static int hc_kick_named_user(const char *kicking_nickname, const char *nickname
 	gboolean is_created=FALSE;
 
 	CHUNK_CORE *cc;
-	int temp_ban;
 
 	HL_LOCK_READ(cluster_entry_lock);
 	hce=find_hub_cnx_entry_with_a_user(nickname);
@@ -1360,19 +1507,63 @@ static int hc_kick_named_user(const char *kicking_nickname, const char *nickname
 	cc=chunk_core_new(CC_USER_KICK);
 	chunk_core_append_subchunk(cc,SC_USER_NICK,strlen(nickname),nickname);
 	
-	/* retrieve the temp ban duration when a kick occurs */
-	G_LOCK(conf_file);
-	if(!e_db_int_get(conf_file,"KICKTEMPBAN",&temp_ban))
-		temp_ban=0;
-	G_UNLOCK(conf_file);
+	if (kicking_nickname!=NULL)
+		chunk_core_append_subchunk(cc,SC_KICK_KICKER,strlen(kicking_nickname),kicking_nickname);
 
-	if(temp_ban)
+	/* send the chunk to hce */
+	bin_xfio_append_new_gba_to_glist_outgoing(hce->hub_bxfio,
+											  chunk_convert_chunk_to_mem(cc),
+											  FALSE);	/* dont copy the chunk, don't free gba */
+	/* free the chunk */
+	chunk_free(cc);
+
+	HL_UNLOCK_READ(cluster_entry_lock);
+	return 0;
+}
+
+/****************************************/
+/* ban a user having the given nickname */
+/****************************************/
+/* output: 0=ok else user not found */
+/************************************/
+static int hc_ban_named_user(const char *kicking_nickname, const char *nickname, unsigned long duration)
+{
+	HUB_CNX_ENTRY *hce;
+	HUB_USER_ENTRY *hue;
+	gboolean is_created=FALSE;
+
+	CHUNK_CORE *cc;
+
+	HL_LOCK_READ(cluster_entry_lock);
+	hce=find_hub_cnx_entry_with_a_user(nickname);
+	if(hce==NULL)
 	{
-		guint32 ntemp_ban;
-		ntemp_ban=GUINT32_TO_BE(temp_ban);
-		chunk_core_append_subchunk(cc,SC_KICK_DURATION,sizeof(ntemp_ban),&ntemp_ban);
+		HL_UNLOCK_READ(cluster_entry_lock);
+		return 1;	/* not found */
 	}
 
+	hue=find_user_named_for_this_hub(hce,nickname,strlen(nickname),&is_created,FALSE);	/* no creation of missing user */
+	if(hue==NULL)
+	{
+		fprintf(stderr,"hc_ban_named_user: internal error, a user has disappeared while the lock is set.\n");
+		HL_UNLOCK_READ(cluster_entry_lock);
+		return 1;		/* user not found */
+	}
+
+	if (hue->ext_flag & SHILD_MODE)
+	{
+		HL_UNLOCK_READ(cluster_entry_lock);
+		return 0;	/* user found, nothing to do */
+	}
+
+	cc=chunk_core_new(CC_USER_BAN);
+	chunk_core_append_subchunk(cc,SC_USER_NICK,strlen(nickname),nickname);
+	{
+		guint32 ntemp_ban;
+		ntemp_ban=GUINT32_TO_BE(duration);
+		chunk_core_append_subchunk(cc,SC_KICK_DURATION,sizeof(ntemp_ban),&ntemp_ban);
+	}
+	
 	if(kicking_nickname!=NULL)
 	{
 		chunk_core_append_subchunk(cc,SC_KICK_KICKER,strlen(kicking_nickname),kicking_nickname);
@@ -1389,6 +1580,86 @@ static int hc_kick_named_user(const char *kicking_nickname, const char *nickname
 	return 0;
 }
 
+/**************************************************************/
+/* ban a user having the given IP                             */
+/* NOTE: this ban is communicated to all nodes of the cluster */
+/**************************************************************/
+/* output: 0=ok else user not found */
+/************************************/
+static int hc_ban_ip(const char *kicking_nickname, struct in_addr ip, unsigned long duration)
+{
+	CHUNK_CORE *cc;
+	GByteArray *gba;
+	CLUSTER_ENTRY *ce;
+	int num_cluster,num_hub;
+
+	HL_LOCK_READ(cluster_entry_lock);
+	cc=chunk_core_new(CC_USER_BANIP);
+	chunk_core_append_subchunk(cc,SC_USER_IP,sizeof(ip.s_addr),&(ip.s_addr));
+	{
+		guint32 ntemp_ban;
+		ntemp_ban=GUINT32_TO_BE(duration);
+		chunk_core_append_subchunk(cc,SC_KICK_DURATION,sizeof(ntemp_ban),&ntemp_ban);
+	}
+	
+	if(kicking_nickname!=NULL)
+	{
+		chunk_core_append_subchunk(cc,SC_KICK_KICKER,strlen(kicking_nickname),kicking_nickname);
+	}
+
+	/* convert the chunk into array */
+	gba=chunk_convert_chunk_to_mem(cc);
+
+	/* now, we must send the packets to all other nodes */
+	num_cluster=0;
+	while((ce=g_list_nth_data(cluster_entry_list,num_cluster++))!=NULL)
+	{
+		num_hub=0;
+		while(num_hub<ce->hc_array->len)
+		{
+			/* send the chunk to hce */
+			bin_xfio_append_new_gba_to_glist_outgoing(g_array_index(ce->hc_array,HUB_CNX_ENTRY,num_hub).hub_bxfio,
+												gba,
+												TRUE);	/* copy the chunk */
+		
+			num_hub++;
+		}
+	}
+
+	/* free the array */
+	g_byte_array_free(gba,TRUE);
+
+	/* free the chunk */
+	chunk_free(cc);
+
+	HL_UNLOCK_READ(cluster_entry_lock);
+	return 0;
+}
+
+/******************************************/
+/* unban a user having the given nickname */
+/******************************************/
+/* output: 0=ok else user not found */
+/************************************/
+static int hc_uban_named_user(const char *nickname)
+{
+	GByteArray *gba;
+
+	CHUNK_CORE *cc;
+
+	cc=chunk_core_new(CC_USER_UBAN);
+	chunk_core_append_subchunk(cc,SC_USER_NICK,strlen(nickname),nickname);
+
+	/* send the chunk to hce */
+	gba=chunk_convert_chunk_to_mem(cc);
+	send_chunk_to_all_hubs(gba);	/* don't free gba */
+
+	/* free the chunk */
+	chunk_free(cc);
+
+	return 0;
+}
+
 /**************************************************************/
 /* function called after a user was disconnected from the hub */
 /* the function is not here to destroy a user itself, it is   */
@@ -1528,6 +1799,9 @@ static GLOBAL_USER_CALLBACKS hub_cnx_entry_glus=
 						hc_glus_get_nick_lists,             /* internal */
 						hc_add_hub_users_stat,              /* internal */
 						hc_kick_named_user,                 /* -> CC_USER_KICK */
+						hc_ban_named_user,                  /* -> CC_USER_BAN */
+						hc_ban_ip,                  				/* -> for ban by ip (to be code) */
+						hc_uban_named_user,                 /* -> CC_USER_UBAN */
 						hc_disconnect_named_user,           /* -> CC_USER_DISC */
 						hc_user_is_disconnected,            /* nothing to do */
 						hc_user_do_hello,                   /* -> CC_USER_IN */
