@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 /*
-$Id: main.c,v 2.36 2003/05/29 13:20:06 ericprev Exp $
+$Id: main.c,v 2.56 2003/11/22 13:47:08 blusseau Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -47,7 +47,6 @@ $Id: main.c,v 2.36 2003/05/29 13:20:06 ericprev Exp $
 #include <sys/wait.h>
 
 #include <glib.h>
-#include <passwd.h>
 #include <regex.h>
 
 #ifdef WIN32
@@ -97,16 +96,10 @@ $Id: main.c,v 2.36 2003/05/29 13:20:06 ericprev Exp $
 #ifdef WITH_PERL
 	#include "emb_perl.h"
 #endif
-#ifdef WITH_PYTHON
-	#include "emb_python.h"
-#endif
 #include "timed_out_string.h"
 #include "tos_key.h"
 #include "color_debug.h"
 #include "gvar.h"
-#include "seen_list.h"
-#include "afk_list.h"
-#include "msg_list.h"
 #include "plugin.h"
 #include "ged.h"
 #include "main_cnx.h"
@@ -124,6 +117,10 @@ $Id: main.c,v 2.36 2003/05/29 13:20:06 ericprev Exp $
 #include "hub_passwd.h"
 #include "lmp.h"
 #include "md_db.h"
+#include "xmltools.h"
+#include "users_xml.h"
+#include "db_xml.h"
+#include "mmap_tools.h"
 
 #ifdef sun
 	#define HAVE_NO_THREAD 1
@@ -148,6 +145,14 @@ $Id: main.c,v 2.36 2003/05/29 13:20:06 ericprev Exp $
 char	*llunsigned_to_str(guint64 a);
 #endif
 
+#ifdef HAVE_LIBDMALLOC
+#include <dmalloc.h>    /* Gray Watson's library */
+#define show_alloc() dmalloc_log_unfreed()
+#define show_stats() dmalloc_log_stats()
+#else
+#define show_alloc()    /* nothing */
+#define show_stats()    /* nothing */
+#endif
 
 time_t gl_cur_time;
 unsigned short hub_port=411;
@@ -155,8 +160,9 @@ unsigned short reg_hub_port=0;
 
 unsigned short cluster_port=410;		/* default cluster port */
 
-GString	*hubname;
+GString	*hubname=NULL;
 char	*tos_filename=NULL;
+char	*dchub_templatedir=NULL;
 int	gl_sr_interval;
 int	gl_moderate_mode;
 int	gl_silent_limit;
@@ -169,7 +175,6 @@ int	gl_resolver_fd = -1;
 int	gl_resolv_id = 0;
 #endif
 
-int	gl_gui_mode = 0;
 int	gl_peek_user = 0;
 
 int	daemonize = 0;
@@ -183,17 +188,13 @@ char	*gl_bind_addr;
 /* configuration file vars */
 /***************************/
 char *conf_filename=NULL;		/* name of the configuration file */
-t_db_sym **conf_file=NULL;
-G_LOCK_DEFINE(conf_file);
 
 /******************/
 /* user file vars */
 /******************/
 char *user_filename=NULL;		/* name of the user file */
-char *user_uid=NULL;				/* name of the user */
+char *user_uid=NULL;			/* name of the user */
 struct passwd *user_pw=NULL;	/* passwd entry containing pw_uid of the user to use */
-FILE *user_file=NULL;
-G_LOCK_DEFINE(user_file);
 
 /*********************/
 /* LOCK_PATTERN vars */
@@ -207,14 +208,6 @@ G_LOCK_DEFINE(gl_lock_patern);
 char *perl_script_dir=NULL;
 char *perl_script_main=NULL;
 
-#ifdef WITH_PYTHON
-/*****************************************/
-/* some vars used for Python interpreter */
-/*****************************************/
-char *python_script_dir  = NULL;
-char *python_main_script = NULL;
-#endif
-
 /*****************************************/
 /* somes vars used for external programs */
 /*****************************************/
@@ -579,6 +572,7 @@ static void start_new_ext_prog(char *progname)
 	int sockfd[2];			/* 0=hub side, 1=program side */
 	LOCAL_USER_CNX_ENTRY *luce;
 	int num;
+	gint16 level;
 	struct my_args param;
 
 
@@ -593,15 +587,19 @@ static void start_new_ext_prog(char *progname)
 		return;
 
 	/* create I/O structure */
+	if (!xml_get_user(progname,&level,NULL,NULL,NULL))	/* look if the bot is register     */
+		level=get_right_level(OPERATOR);				/* and take is level, if not found */
+	                                                    /* use the OPERATOR level          */
 	luce=uc_create_entry(create_xfio(sockfd[0],NULL),		/* create and register an I/O structure */
 	                     progname,		/* nickname */
-	                     0,					/* shared size */
-	                     NULL,				/* mail */
-	                     NULL,				/* description */
-	                     NULL,				/* client version */
-	                     1,					/* user flag */
-	                     0,					/* ext flag */
-	                     TYPE_b_PRIV);	/* it is an OpBot */
+	                     0,				/* shared size */
+	                     NULL,			/* mail */
+	                     NULL,			/* description */
+	                     NULL,			/* client version */
+	                     1,				/* user flag */
+	                     0,				/* ext flag */
+						 level, 		/* default level for a Bot */
+						 TRUE);			/* it's a BOT */
 	                     
 	param.filename = progname;
 	param.fds = sockfd;
@@ -642,9 +640,9 @@ static void start_new_ext_prog(char *progname)
 		glus_do_hello(luce->user_nick->str);
 
 		/* send the myinfo to everyone */
-		glus_do_my_info(	luce->user_nick->str, luce->user_description->str, luce->user_cnx_type,
-				 				luce->user_flag, luce->user_mail->str, luce->shared_size, 
-								luce->ext_flag, luce->privilege, luce->client_version->str);
+		glus_do_my_info(luce->user_nick->str, luce->user_description->str, luce->user_cnx_type,
+						luce->user_flag, luce->user_mail->str, luce->shared_size, 
+						luce->ext_flag, luce->level, luce->client_version->str);
 
 		luce->ext_display_flag|=HAVE_SENT_MYINFO;
 
@@ -773,95 +771,185 @@ static void set_sig(void)
 #endif
 }
 
+static void create_default_conffile(void) {
+	FILE *fxml_conf_file;
+	GString *template_conf_file;
+	size_t flen;
+	unsigned char *fadr;
+	struct stat st;
+
+	template_conf_file=g_string_new("");
+	g_string_sprintf(template_conf_file,"%s/%s",
+					(dchub_templatedir ? dchub_templatedir : DCHUB_TEMPLATEDIR),
+					"conf.xml.in");
+
+	if (stat(template_conf_file->str, &st) != 0) {
+		printf ("Error: template file %s doesn't exist.\n", template_conf_file->str);
+		g_string_free(template_conf_file, TRUE);
+		exit(1);
+	}
+			
+	fxml_conf_file=fopen(conf_filename,"wb");
+	if (fxml_conf_file == NULL) {
+		printf("Can't create configuration file %s: %s\n", conf_filename, strerror(errno));
+		exit(1);
+	}
+				
+	/* copy the template */
+	fadr=map_file(template_conf_file->str, &flen);
+	fwrite(fadr, 1, flen, fxml_conf_file);
+	unmap_file(fadr, flen);
+	fclose(fxml_conf_file);
+	g_string_free(template_conf_file, TRUE);
+	
+	if (!loadDBFile(conf_filename)) {
+		fprintf(stderr,"%s is not a valid configuration file\n",conf_filename);
+		exit(1);
+	}
+
+	/* init the default variable */
+	db_str_set("HUBNAME","noname");		/* hubname */
+	db_str_set("HUBADDR","");			/* address of this server: (for DC Hublist) */
+	db_str_set("HUBDESC","");			/* description of this server (for DC Hublist) */
+
+	db_int_set("REG_HUB",0);			/* the server should be registered on DC Hublist server ? (0=No) */
+	db_str_set("REG_SERV","vandel405.dynip.com");/* address of the DC Hublist server */
+#if 0
+	db_str_set("REG_SERV","dreamland.gotdns.org");/* address of a testing DC Hublist server */
+#endif
+	db_int_set("MAXUSER",100);			/* maximum number of users allowed on this hub */
+	db_int_set("REDIR_FULL",0);			/* when the hub is full, redirect users to another hub ? (0=No) */
+	db_str_set("REDIR_ADDR","");		/* address of the hub to redirect user to */
+#if 0
+	db_str_set("LOCK_PATTERN","^.*");	/* default denied nicknames: all nicknames */
+#endif
+
+	db_int_set("ONLY_REG",0);			/* this is a public hub */
+			      						
+	db_int_set("PUBCHAT",1);			/* public chat deactivated */
+
+	db_int_set("MULTICHAT",1);			/* private chat deactivated */
+
+	db_int_set("MIN_SR_INTERVAL",10);	/* interval deetewn 2 search */
+
+	db_int_set("SPAM_SILENT_LIMIT",10);	/* nbr message needed befor silence the user */
+
+	db_int_set("SPAM_KICK_LIMIT",10);	/* nbr message needed befor diconnect the user */
+
+	db_int_set("BUFFER_SCALE",8);	    /* buffer size in Ko */
+
+	db_int_set("ALL_REDIR", 0);			/* redirect all user */
+}
+
+static void create_default_userfile(void) {
+	FILE *fxml_user_file;
+	GString *template_user_file;
+	size_t flen;
+	unsigned char *fadr;
+	struct stat st;
+
+	template_user_file=g_string_new("");
+	g_string_sprintf(template_user_file,"%s/%s",
+					(dchub_templatedir ? dchub_templatedir : DCHUB_TEMPLATEDIR),
+					"users.xml.in");
+
+	if (stat(template_user_file->str, &st) != 0) {
+		printf ("Error: template file %s doesn't exist.\n", template_user_file->str);
+		g_string_free(template_user_file, TRUE);
+		exit(1);
+	}
+			
+	fxml_user_file=fopen(user_filename,"wb");
+	if (fxml_user_file == NULL) {
+		printf("Can't create user file %s: %s\n", user_filename, strerror(errno));
+		exit(1);
+	}
+				
+	/* copy the template */
+	fadr=map_file(template_user_file->str, &flen);
+	fwrite(fadr, 1, flen, fxml_user_file);
+	unmap_file(fadr, flen);
+	fclose(fxml_user_file);
+	g_string_free(template_user_file, TRUE);
+}
+	
 static void	display_usage(char *prog)
 {
 	fprintf(stderr,"Usage: %s [options]\n"
-						"Options:\n"
-						" -h, --help                      Display this help\n"
-						" -c, --conf=FILENAME             use the given filename as configuration file\n"
-						"                                 (required)\n"
-						" -u, --user=FILENAME             use the given filename as user/passwd file\n"
-						"                                 (required)\n"
-						" -P, --hubpasswd=FILENAME        use the given filename as hub password file\n"
-						"                                 (required)\n"
-						" -r, --crc=FILENAME              use the given filename as database for file CRC\n"
-						"                                 (required)\n"
-						" -U, --UID=USERNAME              ask DcHub to change UID after having bind the\n"
-						"                                 main port.\n"
-						" -i, --init                      create the default configuration and user file\n"
-						"                                 content. You must use this option the first\n"
-						"                                 time you start the hub. Note: This function\n"
-						"                                 deletes existing configuration and user file.\n"
-						" -d, --pscriptdir=DIRECTORY      name of the directory containing perl script.\n"
-						" -s, --pscriptinit=FILENAME      name of the filename loaded when perl starts.\n"
-						" -n, --newport=NUMBER            change listening port of the hub. After using\n"
-						"                                 this option, the new port is stored in the\n"
-						"                                 database and the option is no more required.\n"
-						" -C, --clusterport=NUMBER        change cluster listening port of the hub. After\n"
-						"                                 using this option, the new port is stored in\n"
-						"                                 the database and the option is no more required\n"
-						" -f, --forceport=NUMBER          force DCHUB to register a specific Port\n"
-						" -e, --exprogdir=DIRECTORY       name of the directory containing external\n"
-						"                                 programs.\n"
-						"                                 (NOTE: if a file named 'AUTOSTART' exists in\n"
-						"                                 this directory, this file contains the name of\n"
-						"                                 all external programs to start (1 per line)\n"
-						" -b, --bind=IP                   if the machin have more than one external IP\n"
-#ifdef WITH_PYTHON
-						" -p, --pyscriptdir=DIRECTORY     name of the directory containing Python scripts\n"
-						" -x, --pyinit=FILENAME           name of the filename loaded when Python starts.\n"
-#endif
-						" -l, --linkdir=DIRECTORY         name of the directory containing plugins.\n"
-						"                                 (NOTE: if a file named 'AUTOSTART' exists in\n"
-						"                                 this directory, this file contains the name of\n"
-						"                                 all plugins to load on start (1 per line)\n"
-						" -g, --gui                       gui mode.\n"
-						" -w, --pidfile                   sets the file to which DcHub records the daemon\n"
-						"                                 process id.\n"
-						" -L, --logfile                   sets the file to which DcHub will write is log.\n"
-						" -z, --daemonize                 daemonize DcHub.\n"
+			"Options:\n"
+			" -h, --help                      Display this help\n"
+			" -c, --conf=FILENAME             use the given filename as configuration file\n"
+			" -u, --user=FILENAME             use the given filename as user/passwd file\n"
+			" -P, --hubpasswd=FILENAME        use the given filename as hub password file\n"
+			" -r, --crc=FILENAME              use the given filename as database for file CRC\n"
+			" -U, --UID=USERNAME              ask DcHub to change UID after having bind the\n"
+			"                                 main port.\n"
+			" -i, --init                      create the default configuration and user file\n"
+			"                                 content. You must use this option the first\n"
+			"                                 time you start the hub. Note: This function\n"
+			"                                 deletes existing configuration and user file.\n"
+			" -m, --missing                   create the default configuration and user file\n"
+			"                                 content if the files are missing.\n"
+			" -d, --pscriptdir=DIRECTORY      name of the directory containing perl script.\n"
+			" -s, --pscriptinit=FILENAME      name of the filename loaded when perl starts.\n"
+			" -T, --templatedir=DIRECTORY     name of the directory containing the XML template\n"
+			"                                 files [DEFAULT: %s]\n"
+			" -n, --newport=NUMBER            change listening port of the hub. After using\n"
+			"                                 this option, the new port is stored in the\n"
+			"                                 database and the option is no more required.\n"
+			" -C, --clusterport=NUMBER        change cluster listening port of the hub. After\n"
+			"                                 using this option, the new port is stored in\n"
+			"                                 the database and the option is no more required\n"
+			" -f, --forceport=NUMBER          force DCHUB to register a specific Port\n"
+			" -e, --exprogdir=DIRECTORY       name of the directory containing external\n"
+			"                                 programs.\n"
+			"                                 (NOTE: if a file named 'AUTOSTART' exists in\n"
+			"                                 this directory, this file contains the name of\n"
+			"                                 all external programs to start (1 per line)\n"
+			" -b, --bind=IP                   if the machin have more than one external IP\n"
+			" -l, --linkdir=DIRECTORY         name of the directory containing plugins.\n"
+			"                                 (NOTE: if a file named 'AUTOSTART' exists in\n"
+			"                                 this directory, this file contains the name of\n"
+			"                                 all plugins to load on start (1 per line)\n"
+			" -w, --pidfile                   sets the file to which DcHub records the daemon\n"
+			"                                 process id.\n"
+			" -L, --logfile                   sets the file to which DcHub will write is log.\n"
+			" -z, --daemonize                 daemonize DcHub.\n"
 			
 #ifndef HAVE_GETOPT_LONG
-						"\n  WARNING this build do not support long arguments\n"
-#endif
-						"\n",prog);
+			"\n  WARNING this build do not support long arguments\n"
+#endif	
+			"\n",prog,DCHUB_TEMPLATEDIR);
 }
 
 static void parse_command_line_options(int argc,char **argv)
 {
 #ifdef HAVE_GETOPT_LONG
 	static struct option optab[]= {
-												{"help",no_argument,NULL,'h'},           /* get help */
-												{"conf",required_argument,NULL,'c'},	/* define the configuration file */
-												{"user",required_argument,NULL,'u'},	/* define the user/passwd file */
-												{"hubpasswd",required_argument,NULL,'P'},	/* define the hub passwd file */
-												{"UID",required_argument,NULL,'U'},	/* change UID after running */
-												{"init",no_argument,NULL,'i'},			/* initialize configuration & user/passwd files to their default value */
-												{"gui", no_argument,NULL,'g'},
-												{"pidfile", required_argument,NULL,'w'},
-												{"logfile", required_argument,NULL,'L'},
-												{"daemonize", no_argument,NULL,'z'},
-												{"missing",no_argument,NULL,'m'},		/* add missing variables to the configuration & user/passwd files */
-												{"pscriptdir",required_argument,NULL,'d'},	/* the Perl script dir */
-												{"pscriptinit",required_argument,NULL,'s'},	/* the Perl script containing the main */
-												{"newport",required_argument,NULL,'n'},	/* the new hub port to use */
-												{"clusterport",required_argument,NULL,'C'},	/* the new hub port to use */
-												{"forceport",required_argument,NULL,'f'},	/* the new hub port to reg */
-												{"exprogdir",required_argument,NULL,'e'},	/* the dir containing external program */
-												{"bind",required_argument,NULL,'b'},	/*  */ 
-#ifdef WITH_PYTHON
-												{"pyscriptdir",required_argument,NULL,'p'},
-												{"pyinit",required_argument,NULL,'x'},
-#endif
-												{"linkdir",required_argument,NULL,'l'},	/* the dir containing plugins */
-												{"crc",required_argument,NULL,'r'},	/* define the MD file */
-												{NULL,0,NULL,'\0'}               /* last option */
-											};
-#endif
-	static const char *short_opt="hc:u:igzw:L:md:s:n:C:e:U:f"
-#ifdef WITH_PYTHON
-	                             "p:x:"
+		{"help",no_argument,NULL,'h'},           /* get help */
+		{"conf",required_argument,NULL,'c'},	/* define the configuration file */
+		{"user",required_argument,NULL,'u'},	/* define the user/passwd file */
+		{"hubpasswd",required_argument,NULL,'P'},	/* define the hub passwd file */
+		{"UID",required_argument,NULL,'U'},	/* change UID after running */
+		{"init",no_argument,NULL,'i'},			/* initialize configuration & user/passwd files to their default value */
+		{"pidfile", required_argument,NULL,'w'},
+		{"logfile", required_argument,NULL,'L'},
+		{"daemonize", no_argument,NULL,'z'},
+		{"missing",no_argument,NULL,'m'},		/* add missing variables to the configuration & user/passwd files */
+		{"pscriptdir",required_argument,NULL,'d'},	/* the Perl script dir */
+		{"pscriptinit",required_argument,NULL,'s'},	/* the Perl script containing the main */
+		{"templatedir",required_argument,NULL,'T'},	/* the template dir containing some XML files */
+		{"newport",required_argument,NULL,'n'},	/* the new hub port to use */
+		{"clusterport",required_argument,NULL,'C'},	/* the new hub port to use */
+		{"forceport",required_argument,NULL,'f'},	/* the new hub port to reg */
+		{"exprogdir",required_argument,NULL,'e'},	/* the dir containing external program */
+		{"bind",required_argument,NULL,'b'},	/*  */ 
+		{"linkdir",required_argument,NULL,'l'},	/* the dir containing plugins */
+		{"crc",required_argument,NULL,'r'},	/* define the MD file */
+		{NULL,0,NULL,'\0'}               /* last option */
+	};
 #endif
+	static const char *short_opt="hc:u:igzw:L:md:s:T:n:C:e:U:f:"
 	                             "b:l:P:r:";
 	int do_init=0;
 	int add_missing=0;
@@ -872,11 +960,13 @@ static void parse_command_line_options(int argc,char **argv)
 	int tmp_reg_hub_port=-1;
 	char *ed2kdb_file=NULL;
 
+#if 0
 	if(argc==1)
 	{
 		display_usage(argv[0]);
 		exit(0);
 	}
+#endif
 	opterr = 0;
 #ifdef HAVE_GETOPT_LONG
 	while((ch=getopt_long(argc,argv,short_opt,optab,NULL))!=EOF)
@@ -896,10 +986,6 @@ static void parse_command_line_options(int argc,char **argv)
 							conf_filename=strdup(optarg);
 							break;
 
-			case 'g':
-							gl_gui_mode = 1;
-							break;
-
 			case 'w':
 							if (pidfile!=NULL)
 								free(pidfile);
@@ -971,19 +1057,12 @@ static void parse_command_line_options(int argc,char **argv)
 							perl_script_main=strdup(optarg);
 							break;
 
-#ifdef WITH_PYTHON
-			case 'p':
-							if ( python_script_dir != NULL )
-								free( python_script_dir );
-							python_script_dir = strdup( optarg );
+			case 'T':
+							if(dchub_templatedir != NULL)
+								free(dchub_templatedir);
+							dchub_templatedir=strdup(optarg);
 							break;
 
-			case 'x':
-							if ( python_main_script != NULL )
-								free( python_main_script );
-							python_main_script = strdup( optarg );
-							break;
-#endif
 			case 'l':
 							if ( plugin_directory != NULL )
 								free( plugin_directory );
@@ -1031,15 +1110,21 @@ static void parse_command_line_options(int argc,char **argv)
 		}
 	}
 
+	/* init XML Tools */
+	if (!initXMLTools(dchub_templatedir)) {
+		exit(1);
+	}
+
 	/* check if required variables are filled ... and filled with correct values */
 
 	/* ------------------------------- */
 	/* 1) check the configuration file */
 	/* ------------------------------- */
-	if(conf_filename==NULL)
-	{
-		fprintf(stderr,"You must specify a configuration file.\n");
-		exit(1);
+	if(conf_filename==NULL) {
+		GString* default_conf_filename=g_string_new("");
+		g_string_sprintf(default_conf_filename,"%s/%s",DCHUB_DBDIR,"conf.xml");
+		conf_filename=strdup(default_conf_filename->str);
+		g_string_free(default_conf_filename, TRUE);
 	}
 
 	if(stat(conf_filename,&st)==0) 
@@ -1054,7 +1139,7 @@ static void parse_command_line_options(int argc,char **argv)
 	{
 		if((do_init|add_missing)==0)
 		{
-			fprintf(stderr,"The configuration filename you gave does not exist,\nyou must use --init to initialize it.\n");
+			fprintf(stderr,"The configuration filename you gave (%s) does not exist,\nyou must use --init or --missing to create it.\n",conf_filename);
 			exit(1);
 		}
 	}
@@ -1083,8 +1168,10 @@ static void parse_command_line_options(int argc,char **argv)
 	/* ------------------------------- */
 	if(user_filename==NULL)
 	{
-		fprintf(stderr,"You must specify a user file.\n");
-		exit(1);
+		GString* default_user_filename=g_string_new("");
+		g_string_sprintf(default_user_filename,"%s/%s",DCHUB_DBDIR,"users.xml");
+		user_filename=strdup(default_user_filename->str);
+		g_string_free(default_user_filename, TRUE);
 	}
 
 	if(stat(user_filename,&st)==0) 
@@ -1099,7 +1186,7 @@ static void parse_command_line_options(int argc,char **argv)
 	{
 		if((do_init|add_missing)==0)
 		{
-			fprintf(stderr,"The user filename you gave does not exist,\nyou must use --init to initialize it.\n");
+			fprintf(stderr,"The user filename you gave does not exist,\nyou must use --init or --missing to create it.\n");
 			exit(1);
 		}
 	}
@@ -1109,8 +1196,10 @@ static void parse_command_line_options(int argc,char **argv)
 	/* ------------------------------- */
 	if(hub_passwd_file==NULL)
 	{
-		fprintf(stderr,"You must specify a hub password file.\n");
-		exit(1);
+		GString* default_passwd_filename=g_string_new("");
+		g_string_sprintf(default_passwd_filename,"%s/%s",DCHUB_DBDIR,"dchub.hubpasswd");
+		hub_passwd_file=strdup(default_passwd_filename->str);
+		g_string_free(default_passwd_filename, TRUE);
 	}
 
 	if(stat(hub_passwd_file,&st)==0) 
@@ -1134,8 +1223,10 @@ static void parse_command_line_options(int argc,char **argv)
 	/* -------------------------- */
 	if(ed2kdb_file==NULL)
 	{
-		fprintf(stderr,"You must specify a CRC filename.\n");
-		exit(1);
+		GString* default_ed2kdb_filename=g_string_new("");
+		g_string_sprintf(default_ed2kdb_filename,"%s/%s",DCHUB_DBDIR,"dchub.crc");
+		ed2kdb_file=strdup(default_ed2kdb_filename->str);
+		g_string_free(default_ed2kdb_filename, TRUE);
 	}
 
 	if(stat(ed2kdb_file,&st)==0) 
@@ -1173,88 +1264,11 @@ static void parse_command_line_options(int argc,char **argv)
 		set_user_id();
 		
 	    /* 2nd: create the default configuration file */
-		conf_file=e_db_open(conf_filename);
-		if(conf_file==NULL)
-		{
-			fprintf(stderr,"Unable to create %s\n",conf_filename);
-			exit(1);
-		}
-
-		/* init the default variable */
-		e_db_str_set(conf_file,"HUBNAME","noname");		/* hubname */
-		e_db_str_set(conf_file,"HUBADDR","");				/* address of this server: (for DC Hublist) */
-		e_db_str_set(conf_file,"HUBDESC","");				/* description of this server (for DC Hublist) */
-
-		e_db_int_set(conf_file,"REG_HUB",0);				/* the server should be registered on DC Hublist server ? (0=No) */
-		e_db_str_set(conf_file,"REG_SERV","vandel405.dynip.com");	/* address of the DC Hublist server */
-#if 0
-		e_db_str_set(conf_file,"REG_SERV","dreamland.gotdns.org");	/* address of a testing DC Hublist server */
-#endif
-		e_db_int_set(conf_file,"MAXUSER",100);				/* maximum number of users allowed on this hub */
-		e_db_int_set(conf_file,"REDIR_FULL",0);			/* when the hub is full, redirect users to another hub ? (0=No) */
-		e_db_str_set(conf_file,"REDIR_ADDR","");			/* address of the hub to redirect user to */
-
-#if 0
-		e_db_str_set(conf_file,"LOCK_PATTERN","^.*");	/* default denied nicknames: all nicknames */
-#endif
-
-		e_db_int_set(conf_file,"ONLY_REG",0);			/* this is a public hub */
-
-		e_db_int_set(conf_file,"PUBCHAT",1);			/* public chat deactivated */
-
-		e_db_int_set(conf_file,"MULTICHAT",1);			/* private chat deactivated */
-
-		e_db_int_set(conf_file,"KICKTEMPBAN",0);		/* default ban time */
-
-		e_db_int_set(conf_file,"MIN_SR_INTERVAL",10);		/* interval deetewn 2 search */
-
-		e_db_int_set(conf_file,"SPAM_SILENT_LIMIT",10);		/* nbr message needed befor silence the user */
-
-		e_db_int_set(conf_file,"SPAM_KICK_LIMIT",10);		/* nbr message needed befor diconnect the user */
-
-		e_db_int_set(conf_file,"BUFFER_SCALE",8);	    /* buffer size in Ko */
-
-		e_db_int_set(conf_file,"ALL_REDIR", 0);		/* redirect all user */
-
-		e_db_close(conf_file); // ok
-		conf_file=NULL;
-		e_db_flush();
-
+		create_default_conffile();
+		
 		/* 3rd: create the default user/passwd file */
-		user_file=fopen(user_filename,"wb");
-		if(user_file==NULL)
-		{
-			fprintf(stderr,"Unable to create %s\n",user_filename);
-			exit(1);
-		}
-
-		/* user file format is quite simple: */
-		/* ABBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBCCCCCCCCCCCCCD */
-		/* where A='F' if the entry is empty and 'U' if it is used */
-		/* B...B is a 32 bytes C-string (always ended by a '\0') containing a nickname. */
-		/*        => the longest nickname takes 31 chars */
-		/* C...C is a 16 butes C-string (always ended by a '\0') containing a password. */
-		/*        => the longest password takes 15 chars */
-		/* D is a 1 byte value: D='O' if this nick is an operator, D='N' if this nick is a normal user */
-		/*        D='M' if this nick is the master of the hub (==operator + special function) */
-		/*        only the master of the hub can modify configuration file remotely */
-		{
-			char master_user[]={'U',
-									  'M','A','S','T','E','R',0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
-									  'M','A','S','T','E','R',0,0,0,0,0,0,0,0,0,0,
-									  'M'};
-
-			if(fwrite(master_user,1,sizeof(master_user),user_file)!=sizeof(master_user))
-			{
-				fclose(user_file); // ok
-				unlink(user_filename);
-				fprintf(stderr,"Unable to write inside %s. Disk full ? \n",user_filename);
-				exit(1);
-			}
-		}
-
-		fclose(user_file); // ok
-		user_file=NULL;
+		create_default_userfile();
+		
 		fprintf(stderr,"Init done. Check the configuration file if you want to modify\n"
 							"it. NEVER edit user file. After, you can start the hub without\n"
 							"the --init option.\n"
@@ -1272,7 +1286,6 @@ static void parse_command_line_options(int argc,char **argv)
 							"ONLY_REG => '0'\n"
 							"PUBCHAT => '1'\n"
 							"MULTICHAT => '1'\n"
-							"KICKTEMPBAN => '0'\n"
 							"MIN_SR_INTERVAL => '10'\n"
 							"SPAM_SILENT_LIMIT => '10'\n"
 							"SPAM_KICK_LIMIT => '10'\n"
@@ -1292,6 +1305,20 @@ static void parse_command_line_options(int argc,char **argv)
 		exit(0);
 	}
 
+	if (add_missing == 1 ) {
+
+		set_user_id();
+
+		if (stat(conf_filename, &st) != 0) {
+			/* create a default conf file */
+			create_default_conffile();
+		}
+		if (stat(user_filename, &st) != 0) {
+			/* create a default users file */
+			create_default_userfile();
+		}
+	}
+
 	/* we are ready to start the hub */
 	if (daemonize) {
 		int child;
@@ -1346,10 +1373,8 @@ static void parse_command_line_options(int argc,char **argv)
 	}
 	
 	/* open the configuration file */
-	conf_file=e_db_open(conf_filename);
-	if(conf_file==NULL)
-	{
-		fprintf(stderr,"Unable to open %s\n",conf_filename);
+	if (!loadDBFile(conf_filename)) {
+		fprintf(stderr,"%s is not a valid configuration file\n",conf_filename);
 		exit(1);
 	}
 
@@ -1357,29 +1382,27 @@ static void parse_command_line_options(int argc,char **argv)
 	{
 		/* if a port is provided, save the port number into the database */
 		printf ("update HUBPORT to %d\n", new_hub_port);
-		e_db_int_set(conf_file,"HUBPORT",new_hub_port);
+		db_int_set("HUBPORT",new_hub_port);
 	}
 
 	if(new_cluster_port!=-1)
 	{
 		printf ("update CLUSTERPORT to %d\n", new_cluster_port);
-		e_db_int_set(conf_file,"CLUSTERPORT",new_cluster_port);
+		db_int_set("CLUSTERPORT",new_cluster_port);
 	}
-
-
+			
 	/* initialize hub_port param to the saved value or to the default 411 value */
-	if(!e_db_int_get(conf_file,"HUBPORT",&new_hub_port))
+	if(!db_int_get("HUBPORT", &new_hub_port))
 		new_hub_port=411;
-	if(!e_db_int_get(conf_file,"CLUSTERPORT",&new_cluster_port))
+
+	if(!db_int_get("CLUSTERPORT",&new_cluster_port))
 		new_cluster_port=410;
 	hub_port=new_hub_port;
 	cluster_port=new_cluster_port;
 
 	/* open the user file */
-	user_file=fopen(user_filename,"r+b");
-	if(user_file==NULL)
-	{
-		fprintf(stderr,"Unable to open %s\n",user_filename);
+	if (!loadUsersFile(user_filename)) {
+		fprintf(stderr,"%s is not a valid users file\n",user_filename);
 		exit(1);
 	}
 
@@ -1438,21 +1461,24 @@ int load_dyn_conf(void)
 	char	*hub_name;
 	char	*pattern_list;
 
-	G_LOCK(conf_file);
-	hub_name = e_db_str_get(conf_file,"HUBNAME");
-	if (!e_db_int_get(conf_file,"MIN_SR_INTERVAL" , &gl_sr_interval))
+	hub_name=db_str_get("HUBNAME");
+
+	if (!db_int_get("REG_HUB",&do_hub_registration))
+		do_hub_registration=0;
+
+	if (!db_int_get("MIN_SR_INTERVAL" , &gl_sr_interval))
 	{
 		gl_sr_interval = 10;
 	}
-	if (!e_db_int_get(conf_file,"SPAM_SILENT_LIMIT" , &gl_silent_limit))
+	if (!db_int_get("SPAM_SILENT_LIMIT" , &gl_silent_limit))
 	{
 		gl_silent_limit = 10;
 	}
-	if (!e_db_int_get(conf_file,"SPAM_KICK_LIMIT" , &gl_kick_limit))
+	if (!db_int_get("SPAM_KICK_LIMIT" , &gl_kick_limit))
 	{
 		gl_kick_limit = 10;
 	}
-	if (!e_db_int_get(conf_file,"BUFFER_SCALE" , &gl_buf_size))
+	if (!db_int_get("BUFFER_SCALE" , &gl_buf_size))
 	{
 		gl_buf_size = 8;
 	}
@@ -1460,7 +1486,7 @@ int load_dyn_conf(void)
 		gl_buf_size = 2;
 	gl_buf_size *= 1024;
 	
-	if ((pattern_list = e_db_str_get(conf_file,"LOCK_PATTERN")) != NULL)
+	if ((pattern_list = db_str_get("LOCK_PATTERN")) != NULL)
 	{
 		char *tmp;
 		G_LOCK(gl_lock_patern);
@@ -1488,7 +1514,6 @@ int load_dyn_conf(void)
 		}
 		G_UNLOCK(gl_lock_patern);
 	}
-	G_UNLOCK(conf_file);
 
 	if (hubname != NULL)
 		g_string_free(hubname, TRUE);	
@@ -1500,6 +1525,7 @@ int load_dyn_conf(void)
 		free(hub_name);
 	}
 	hubname=g_string_append(hubname,"|");
+	
 	return (0);
 }
 
@@ -1554,22 +1580,13 @@ static void main_loop_tasks_always1(const struct ged_callbacks *ged)
 	}
 	else
 	{
-		int	do_hub_registration = 0;
-
-		G_LOCK(conf_file);
-		e_db_int_get(conf_file,"REG_HUB",&do_hub_registration);
-		G_UNLOCK(conf_file);
 		if (do_hub_registration)
 		{
-			if(last_reg_time + (DELAY_BETWEEN_REGISTRATION-60) < gl_cur_time)
-			{
+			if((last_reg_time + (DELAY_BETWEEN_REGISTRATION-60)) < gl_cur_time) {
 				char	*reg_serv = NULL;
 
-				G_LOCK(conf_file);
-				reg_serv=e_db_str_get(conf_file,"REG_SERV");
-				G_UNLOCK(conf_file);
-				if (reg_serv)
-				{
+				reg_serv=db_str_get("REG_SERV");
+				if (reg_serv) {
 					try_new_registration(reg_serv); /* start a registration */
 					last_reg_time = gl_cur_time;
 					free(reg_serv);
@@ -1588,8 +1605,6 @@ static void main_loop_tasks_always1(const struct ged_callbacks *ged)
 static void main_loop_tasks_periodic(const GED_CALLBACKS *ged)
 {
 	timeout_tos();
-	timeout_afk();
-	timeout_msg();
 	check_prog_to_start();
 
 	/* for even lower priority tasks, only 6 times slower (1 every minute) */
@@ -1604,6 +1619,8 @@ static void main_loop_tasks_periodic(const GED_CALLBACKS *ged)
 			update_dynamic_if_ip();
 		}
 	}
+
+	g_mem_profile();
 }
 
 static GED_CALLBACKS main_loop_ged={
@@ -1696,27 +1713,18 @@ int main(int argc,char **argv)
 	/* handler for the tasks of the main loop */
 	ged_register(manage_main_loop_tasks_init,999);		/* lowest priority */
 
-#if 1
-#warning code disabled (gui)
-#else
-	if(gl_gui_mode)
-		ged_register(manage_gl_gui_init,998);	/* low priority */
-#endif
-
 	set_user_id();
 
 #ifdef WITH_PERL
 	init_perl();
 #endif
 
-#ifdef WITH_PYTHON
-	init_python();
-#endif
-
 #ifdef HAVE_LIBDL
 	load_all_autostart_plugin();
 #endif
 
+	SEND_EVT_TO("hubinit","hubinit",0);
+
 	/* remove in dchub 0.2.5 */
 #if 0
 #ifndef HAVE_NO_THREAD
@@ -1781,37 +1789,26 @@ int main(int argc,char **argv)
 			ged_always();
 		}
 	} /* end of the main loop */
-	if (gl_gui_mode)
-		printf("_end:\n");
-	else
-		printf("ok\n");
+
+	printf("ok\n");
 	
 	/***************************************************/
 	/*************** END close soft ********************/ 
 	/***************************************************/
 
-	if(conf_file!=NULL)
-	{
-		G_LOCK(conf_file);
-		e_db_close(conf_file); 
-		conf_file=NULL;
-		e_db_flush();
-	}
+	/* Free all XML resources for users file */
+	xml_free_users_resource();
 
-	if(user_file!=NULL)
-	{
-		G_LOCK(user_file);
-		fclose(user_file); 
-		user_file=NULL;
-	}
+	/* Free all XML resources for configuration file */
+	xml_free_db_resource();
 
+	/* Free XML Tools */
+	freeXMLTools();
+		
 	save_tos_data(tos_filename);
 #ifdef WITH_PERL
 	exit_perl();
 #endif
-#ifdef WITH_PYTHON
-	exit_python_interpreter();
-#endif
 
 	ged_exit();
 
@@ -1823,7 +1820,7 @@ int main(int argc,char **argv)
 	if(pidfile!=NULL) {
 		unlink(pidfile);	
 	}
-	
+
 	exit(0);
 }
 
