@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 /*
-$Id: main_cnx_handshake.c,v 2.30 2003/06/30 17:20:06 ericprev Exp $
+$Id: main_cnx_handshake.c,v 2.45 2003/11/22 07:01:07 ericprev Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -42,10 +42,10 @@ $Id: main_cnx_handshake.c,v 2.30 2003/06/30 17:20:06 ericprev Exp $
 #include "tos_key.h"
 #include "multi_public_chat.h"
 #include "user_cnx_lst.h"
-#include "seen_list.h"
 #include "global_user_if.h"
 #include "mmap_tools.h"
-#include "passwd.h"
+#include "users_xml.h"
+#include "db_xml.h"
 
 static GArray *cnx_in_handshake=NULL;
 
@@ -76,7 +76,7 @@ void cnx_handshake_register_new_fd(int sock_fd, struct in_addr *remote_addr)
 
 	nw.temp_nick=NULL;
 	nw.temp_passwd=NULL;
-	nw.temp_privilege=0;
+	nw.temp_level=0;
 	nw.temp_ext_flag=0;
 
 	g_array_append_val(cnx_in_handshake,nw);
@@ -127,24 +127,22 @@ static void cnx_handshake_convert_cnx_hshake_to_local_user_cnx_entry(CNX_HSHAKE
 {
 	LOCAL_USER_CNX_ENTRY *luce;
 	int flag;
-	va_list ap;			/* technically, we don't care about the content of this variable but on Alpha, we cannot use NULL as a va_list */
+	va_list ap;	/* technically, we don't care about the content of this variable but on Alpha, */
+	            /* we cannot use NULL as a va_list */
 
-	/* as soon the user is accepted add the nick to the seen list */
-	add_seen_entry("hello",ch->temp_nick,0,ap,NULL);
 	/* send_const_str_to(ptr,"<welcome> This hub is running " PACKAGE " " VERSION". build on "__DATE__" at "__TIME__".|"); */
 
 	/* set the keep-alive flag to detect broken connection */
 	flag=1;
-   setsockopt(ch->xfio->sock_fd, SOL_SOCKET, SO_KEEPALIVE, &flag, sizeof(flag));
+	setsockopt(ch->xfio->sock_fd, SOL_SOCKET, SO_KEEPALIVE, &flag, sizeof(flag));
 
-	luce=uc_create_entry(ch->xfio, ch->temp_nick,0,NULL,NULL,NULL,1,ch->temp_ext_flag,ch->temp_privilege);
+	luce=uc_create_entry(ch->xfio, ch->temp_nick,0,NULL,NULL,NULL,1,ch->temp_ext_flag,ch->temp_level,FALSE);
 
 	/* say hello to everyone */
 	glus_do_hello(luce->user_nick->str);
 
 	/* we still can use ch constant because it is shared with the user_cnx_lst_entry */
-	if(luce->privilege & OP_PRIV)
-	{
+	if(luce->level >= get_right_level(OPERATOR)) {
 		/* if the new user is op, rebuild the oplist and resend it */
 		GString *lists[2];
 		glus_get_nick_lists(&lists[0],&lists[1]);
@@ -159,8 +157,7 @@ static void cnx_handshake_convert_cnx_hshake_to_local_user_cnx_entry(CNX_HSHAKE
 				 "Using this private chat, you can enter special commands:\r\n"
 				 "Use -help for the command list.\r\n|",
 				 luce->user_nick->str,
-				 (luce->privilege & MASTER_PRIV)?"Master of the hub":"operator");
-
+				 (luce->level == MASTER_LEVEL) ? "Master of the hub" : "operator");
 		
 		send_stolen_gstr_to_luce(luce,lists[0]);	/* don't free lists[0] */
 	}
@@ -171,19 +168,16 @@ static void cnx_handshake_convert_cnx_hshake_to_local_user_cnx_entry(CNX_HSHAKE
 		GString *msg;
 
 		/* first, check MOTD key */
-		G_LOCK(conf_file);
-		rd=e_db_str_get(conf_file,"MOTD");
-		G_UNLOCK(conf_file);
+		rd=db_str_get("MOTD");
 
-		if(rd!=NULL)
-		{
+		if (rd != NULL) {
 			size_t flen;
 			unsigned char *fadr;
 
 			fadr=map_file(rd,&flen);
 			free(rd);
 
-			if(fadr==NULL)
+			if (fadr == NULL)
 				goto no_valid_motd;
 
 			msg=g_string_new("");
@@ -194,28 +188,20 @@ static void cnx_handshake_convert_cnx_hshake_to_local_user_cnx_entry(CNX_HSHAKE
 			{
 				int i;
 
-				for(i=0;i<flen;i++)
-				{
+				for (i=0; i<flen; i++)
 					g_string_append_c(msg,fadr[i]);
-				}
 			}
 #endif
 			g_string_append_c(msg,'|');
 			send_stolen_gstr_to_luce(luce,msg);		/* don't free msg */
 
 			unmap_file(fadr,flen);
-		}
-		else
-		{
+		} else {
 			/* if MOTD does not exist or is invalid, check WELCOME key */
-			no_valid_motd:
-			G_LOCK(conf_file);
-			rd=e_db_str_get(conf_file,"WELCOME");
-			G_UNLOCK(conf_file);
-			if(rd!=NULL)
-			{
-				if(strlen(rd)!=0)
-				{
+		  no_valid_motd:
+			rd=db_str_get("WELCOME");
+			if (rd != NULL) {
+				if (strlen(rd) != 0) {
 					msg=g_string_new("");
 					g_string_sprintf(msg,"$To: %s From: Hub $%s|",ch->temp_nick,rd);
 					send_stolen_gstr_to_luce(luce,msg);		/* don't free msg */
@@ -224,7 +210,7 @@ static void cnx_handshake_convert_cnx_hshake_to_local_user_cnx_entry(CNX_HSHAKE
 			}
 		}
 	}
-
+	
 	{
 		unsigned int hip;
 		GString *ip_number;
@@ -333,13 +319,10 @@ static int try_auto_redir(XF_IO *xfio)
 	char *rd;
 	GString *rd_msg;
 
-	G_LOCK(conf_file);
-	rd=e_db_str_get(conf_file,"REDIR_ADDR");
-	G_UNLOCK(conf_file);
-	if(rd==NULL)
+	rd=db_str_get("REDIR_ADDR");
+	if (rd == NULL)
 		return (1);
-	if(strlen(rd)==0)
-	{
+	if (strlen(rd) == 0) {
 		free(rd);
 		return (2);
 	}
@@ -374,29 +357,21 @@ static int too_many_users(XF_IO *xfio)
 
 	cur_nb=user_cnx_entry_get_nb_users();
 	
-	G_LOCK(conf_file);
-	if(!e_db_int_get(conf_file,"MAXUSER",&max_nb))
+	if (!db_int_get("MAXUSER", &max_nb))
 		max_nb=100;
-	G_UNLOCK(conf_file);
 
-	if(cur_nb>=max_nb)
-	{
+	if (cur_nb >= max_nb) {
 		int v;
 		/* too many user connected */
-		G_LOCK(conf_file);
-		if(!e_db_int_get(conf_file,"REDIR_FULL",&v))
+		if (!db_int_get("REDIR_FULL", &v))
 			v=0;
-		G_UNLOCK(conf_file);
 
-		if(v==0)
-		{
+		if (v==0) {
 			/* display an error message */
 			disp_msg:
 			send_const_str_to_xfio(xfio,"<Hub-Security> This hub is currently full!|");
 			send_const_str_to_xfio(xfio,"$HubIsFull|");
-		}
-		else
-		{
+		} else {
 			if (try_auto_redir(xfio) != 0)
 				goto disp_msg;
 			/* can not redirect but the user must not log in */
@@ -408,12 +383,9 @@ static int too_many_users(XF_IO *xfio)
 	{
 		int v;
 
-		G_LOCK(conf_file);
-		if(!e_db_int_get(conf_file,"ALL_REDIR",&v))
+		if(!db_int_get("ALL_REDIR",&v))
 			v=0;
-		G_UNLOCK(conf_file);
-		if (v)
-		{
+		if (v) {
 			if (try_auto_redir(xfio) != 0)
 				goto disp_msg;
 			/* can not redirect but the hub is not full, so let him log in */
@@ -477,12 +449,12 @@ static void sf_step_lock_sent(CNX_HSHAKE *ch)
 static void perform_password_checking(CNX_HSHAKE *ch, LOCAL_USER_CNX_ENTRY *ghost, int only_reg)
 {
 	/* check if this user is a registered user (password protected) */
-	ch->temp_passwd=get_user_password(ch->temp_nick,&(ch->temp_privilege),&(ch->temp_ext_flag));
-
+	xml_get_user(ch->temp_nick, &(ch->temp_level), &(ch->temp_passwd), NULL, NULL);
+	
 	/* check if too many users on the hub */
 	if(only_reg==0)
 	{
-		if((ch->temp_passwd==NULL)||(ch->temp_privilege==0)) /* only normal users (registered or not) are limited in number */
+		if ((ch->temp_passwd==NULL) || (ch->temp_level < get_right_level(OPERATOR))) /* only normal users (registered or not) are limited in number */
 		{
 #if 0
 			if(ghost)
@@ -517,9 +489,9 @@ static void perform_password_checking(CNX_HSHAKE *ch, LOCAL_USER_CNX_ENTRY *ghos
 	}
 
 	if (ch->temp_passwd != NULL)
-	{	/* this is a password protected user */
-		if( (strlen(ch->temp_passwd) > 0) || (ch->temp_privilege> 0) )
-		{ /* OP and MASTER must enter password */
+	{	/* this is a password protected user */		
+		if ((strlen(ch->temp_passwd) > 0) || (ch->temp_level >= get_right_level(OPERATOR)))
+		{	/* OP and MASTER must enter password */
 			/* send password query */
 			send_const_str_to_xfio(ch->xfio,"<Hub-Security> Your name is registered, please supply the password.|");
 			send_const_str_to_xfio(ch->xfio,"$GetPass|");
@@ -571,9 +543,10 @@ static void sf_step_key_valid(CNX_HSHAKE *ch)
 	/* the  character is needed by the regexp lock */
 	/* username with " can not be kick with the Hub-security*/
 	/* dc protocol do not allow username longer than 47 char */
-	if( (strpbrk(ch->temp_nick," $\"")!=NULL) ||
-	    (strlen(ch->temp_nick) > 47) )
-	{
+	if( (strpbrk(ch->temp_nick," <>$\"")!=NULL) ||
+		(strlen(ch->temp_nick) > 47) ||
+		(strlen(ch->temp_nick) < 3) /* at least 2 characters + trailing '|' */
+		) {
 		printf("' ' or '$' or '' in nick.\n");
 		disp_cnx_hshake_info(ch);
 		XFIO_SET_CLOSE_STATUS(ch->xfio,CNX_CLOSED);	/* immediat termination */
@@ -589,10 +562,7 @@ static void sf_step_key_valid(CNX_HSHAKE *ch)
 		char	*dummy1;
 		int	dummy_len;
 
-		if( (strstr(ch->temp_nick,"www.glosearch.tk")) ||
-		    (get_tos_entry(TBAN_TOSKEY,ch->temp_nick,strlen(ch->temp_nick),1,&dummy1,&dummy_len)==1) ||
-		    (get_tos_entry(PBAN_TOSKEY,ch->temp_nick,strlen(ch->temp_nick),1,&dummy1,&dummy_len)==1) )
-		{
+		if (get_tos_entry(BAN_TOSKEY,ch->temp_nick,strlen(ch->temp_nick),1,&dummy1,&dummy_len) == 1) {
 			printf("Connection attempt from a banned nick: %s\n",ch->temp_nick);
 			send_const_str_to_xfio(ch->xfio,"<Hub-Security> Your Nick is banned!!!|");
 			disp_cnx_hshake_info(ch);
@@ -617,10 +587,8 @@ static void sf_step_key_valid(CNX_HSHAKE *ch)
 	}
 
 	/* check if hub is public or not */
-	G_LOCK(conf_file);
-	if(!e_db_int_get(conf_file,"ONLY_REG",&only_reg))
+	if (!db_int_get("ONLY_REG", &only_reg))
 		only_reg=0;
-	G_UNLOCK(conf_file);
 
 	/* check against a ghost user (or another user/chat with the same name) */
 	ghost_luce=user_cnx_entry_get_by_nickname(ch->temp_nick);
@@ -634,13 +602,11 @@ static void sf_step_key_valid(CNX_HSHAKE *ch)
 		}
 		else
 		{
-			if((ghost_luce->privilege==0)&&(only_reg==0))
-			{
+			if ((ghost_luce->level==0) && (only_reg==0) ) {
 				/* not register only and not privilege user but perhaps it is a password protected normal account */
-				char *ghost_luce_passwd;
-				int dummy;
+				char *ghost_luce_passwd=NULL;
 
-				ghost_luce_passwd=get_user_password(ch->temp_nick,&dummy,&dummy);
+				xml_get_user(ch->temp_nick, NULL, &ghost_luce_passwd, NULL, NULL);
 			
 				if(ghost_luce_passwd==NULL)
 				{
@@ -689,6 +655,7 @@ static void sf_step_want_passwd(CNX_HSHAKE *ch)
 	GString *inp;
 	GString *okpass;
 	LOCAL_USER_CNX_ENTRY *ghost_luce;
+	unsigned char passMD5ascii[ENC_MD5_PASSWD_LEN];
 
 	inp=xfio_take_first_string_of_glist_incoming(ch->xfio);
 	if(inp==NULL)
@@ -710,10 +677,13 @@ static void sf_step_want_passwd(CNX_HSHAKE *ch)
 	inp=g_string_erase(inp,0,sizeof("$MyPass ")-1);
 	inp = g_string_truncate(inp, ((inp->len -1) > 0) ? (inp->len -1): 0);
 
-	if(strcmp(inp->str, ch->temp_passwd))
+	/* Encode the password */
+	encrypt_password(inp->str,passMD5ascii);
+	
+    /* Check the password */
+	if(strcmp(passMD5ascii, ch->temp_passwd))
 	{
-		printf("Invalid password (%s) for %s. Wanted: %s\n",
-	 				inp->str,ch->temp_nick,ch->temp_passwd);
+		printf("Invalid password for %s.\n", ch->temp_nick);
 		send_const_str_to_xfio(ch->xfio,"<Hub-Security> Incorrect password.|$BadPass|");
 		g_string_free(inp,TRUE);
 		disp_cnx_hshake_info(ch);
@@ -722,7 +692,6 @@ static void sf_step_want_passwd(CNX_HSHAKE *ch)
 	}
 
 #if 0
-	if (gl_gui_mode == 0)
 	{
 		unsigned int hip;
 
