@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 /*
-$Id: multi_public_chat.c,v 2.19 2003/05/08 08:57:49 ericprev Exp $
+$Id: multi_public_chat.c,v 2.25 2003/11/05 18:16:19 ericprev Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -35,6 +35,8 @@ $Id: multi_public_chat.c,v 2.19 2003/05/08 08:57:49 ericprev Exp $
 #include "global_user_if.h"
 #include "hl_locks.h"
 #include "toolkit.h"
+#include "db_xml.h"
+#include "users_xml.h"
 
 static GHashTable *mpuc_entry_list=NULL;
 
@@ -576,7 +578,7 @@ static GLUS_USER_INFO *mpuc_get_user_info_from_mpuc_entry(MPUC_ENTRY *mpuc)
 		gui->user_mail=g_string_new("");
 		gui->user_flag=1;			/* default flag: 1 */
 		gui->ext_flag=0;			/* default flag: 0 */
-		gui->privilege=0;			/* no privilege */
+		gui->level=0;				/* no level */
 		gui->client_version=g_string_new("");
 		gui->user_ip.s_addr=0;		/* no meaning */
 	}
@@ -666,14 +668,6 @@ static void mpuc_get_nick_lists(GString **lists)
 	HL_UNLOCK_READ(mpuc_entry_list_mutex);
 }
 
-/****************************************************************/
-/* add the shared size and number of users to current variables */
-/****************************************************************/
-static void mpuc_add_hub_users_stat(guint64 *shared_size, unsigned int *nb_users)
-{
-#warning code to write ?
-}
-
 /******************************************/
 /* kick the user having the given name    */
 /* the user is kicked by kicking_nickname */
@@ -753,8 +747,11 @@ static GLOBAL_USER_CALLBACKS mpuc=
 									mpuc_get_users_info,
 									mpuc_get_users_info_by_ip,
 									mpuc_get_nick_lists,
-									mpuc_add_hub_users_stat,
+									NULL,	// mpuc_add_hub_users_stat,
 									mpuc_kick_named_user,
+									NULL,	// mpuc_ban_named_user,
+									NULL,	// mpuc_ban_ip,
+									NULL,	// mpuc_uban_named_user,
 									mpuc_disconnect_named_user,
 									mpuc_user_is_disconnected,
 									NULL,	// mpuc_user_do_hello
@@ -898,7 +895,7 @@ int cmd_create_public_chat(const char *cmd, LOCAL_USER_CNX_ENTRY *luce, GString
                const GString *str, char *xtra_param, const GPtrArray *splitted_str)
 {
 	int i;
-	int ret;
+	gboolean ret;
 	int multi_allow;
 	int chat_id;
 
@@ -912,12 +909,9 @@ int cmd_create_public_chat(const char *cmd, LOCAL_USER_CNX_ENTRY *luce, GString
 	/* xtra_param= NULL                                                     */
 	/* splitted_str = array of strings. It is 'str' splitted along ' '      */
 	/************************************************************************/
-	G_LOCK(conf_file);
-	ret=e_db_int_get(conf_file,"MULTICHAT",&multi_allow);
-	G_UNLOCK(conf_file);
+	ret=db_int_get("MULTICHAT", &multi_allow);
 
-	if(ret==0)
-	{
+	if (ret==0) {
 		g_string_sprintfa((*dest_base),"MULTICHAT key (int) does not exist, multichat creation is disabled\r\n|");
 		send_const_str_to_luce(luce,(*dest_base)->str);	/* we cannot use *dest_base because its reference cannot be stolen */
 		return 0;
@@ -926,21 +920,20 @@ int cmd_create_public_chat(const char *cmd, LOCAL_USER_CNX_ENTRY *luce, GString
 	switch(multi_allow)
 	{
 		case 0:	/* disabled for everyone */
-					g_string_sprintfa((*dest_base),"multichat creation is disabled\r\n|");
-					send_const_str_to_luce(luce,(*dest_base)->str);	/* we cannot use *dest_base because its reference cannot be stolen */
-					return 0;
+			g_string_sprintfa((*dest_base),"multichat creation is disabled\r\n|");
+			send_const_str_to_luce(luce,(*dest_base)->str);	/* we cannot use *dest_base because its reference cannot be stolen */
+			return 0;
 		
 		case 1:	/* op and master only */
-					if (!(luce->privilege & OP_PRIV))
-					{
-						g_string_sprintfa((*dest_base),"multichat creation is only available to Op users\r\n|");
-						send_const_str_to_luce(luce,(*dest_base)->str);	/* we cannot use *dest_base because its reference cannot be stolen */
-						return 0;
-					}
-					break;
+			if (luce->level < get_right_level(OPERATOR)) {
+				g_string_sprintfa((*dest_base),"multichat creation is only available to Op users\r\n|");
+				send_const_str_to_luce(luce,(*dest_base)->str);	/* we cannot use *dest_base because its reference cannot be stolen */
+				return 0;
+			}
+			break;
 	
 		case 2:	/* allowed for everyone */
-					break;
+			break;
 	}
 	
 	i=splitted_str->len;
