@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 /*
-$Id: plugin.c,v 2.4 2003/02/16 11:09:03 eric Exp $
+  $Id: plugin.c,v 2.11 2003/11/28 14:34:56 ericprev Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -37,7 +37,8 @@ $Id: plugin.c,v 2.4 2003/02/16 11:09:03 eric Exp $
 #include "pluginT.h"
 #include "../plugin/dchub_plugin_api.h"
 #include "gvar.h"
-#include "passwd.h"
+#include "users_xml.h"
+#include "db_xml.h"
 #include "global_user_if.h"
 
 static GPtrArray *loaded_plugins=NULL;	/* array of PLUGIN entries */
@@ -440,18 +441,17 @@ int plugin_nick_duration(const char *nickname)
 /******************************************/
 /* get the type of account used by a user */
 /******************************************/
-char plugin_nick_type(const char *nickname)
+gint16 plugin_nick_type(const char *nickname)
 {
-	char	type[1];
+	gint16 result;
 
-	if(nickname==NULL)
-	{
+	if (nickname == NULL) {
 		fprintf(stderr,"plugin_nick_type: invalid input parameters\n");
-		return '\0';
+		return 0;
 	}
-	if (get_account_type(nickname, type) == 0)
-		return ('U');		/* unregistered user */
-	return(*type);
+	if (!xml_get_user(nickname, &result, NULL, NULL, NULL))
+		result=0; /* if user is not found return level 0 */
+	return result;
 }
 
 /**********************************/
@@ -470,10 +470,8 @@ int get_max_users(void)
 {
 	int max_nb;
 
-	G_LOCK(conf_file);
-	if(!e_db_int_get(conf_file,"MAXUSER",&max_nb))
+	if (!db_int_get("MAXUSER", &max_nb))
 		max_nb=100;
-	G_UNLOCK(conf_file);
 
 	return max_nb;
 }
@@ -594,6 +592,7 @@ void plugin_stop(char *plugin_name, GString *output)
 
 	/* and free its entry */
 	delete_plugin_entry_by_pointer(plug);
+	g_string_sprintfa(output,"Ok");
 	return;
 }
 
@@ -646,9 +645,9 @@ void load_all_autostart_plugin(void)
 	g_string_free(str,TRUE);
 }
 
-/**************************************************************************/
-/* send event to plugin (same prototype as perl and python event handler) */
-/**************************************************************************/
+/***************************************************************/
+/* send event to plugin (same prototype as perl event handler) */
+/***************************************************************/
 void send_evt_to_plugin(const char *evt_name, const char *evt_emitter, int nb_args, ...)
 {
 	GPtrArray *gpa;
