@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 /*
-$Id: process_ucmd.c,v 2.35 2003/06/03 22:02:05 blusseau Exp $
+$Id: process_ucmd.c,v 2.45 2003/11/22 07:01:07 ericprev Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -30,20 +30,28 @@ $Id: process_ucmd.c,v 2.35 2003/06/03 22:02:05 blusseau Exp $
 #include <netinet/in.h>
 #include <sys/socket.h>
 #include <ctype.h>
+#ifdef WITH_GCRYPT
+#include <gcrypt.h>
+#define MD4_DIGEST_LENGTH 16
+#define MD4_ALGO(source,len,dest)   gcry_md_hash_buffer(GCRY_MD_MD4,dest,source,len)
+#else
 #include <openssl/md4.h>
+#define MD4_ALGO MD4
+#endif
 #include <glib.h>
 
 #include "process_ucmd.h"
 #include "user_cnx_lst.h"
 #include "macro.h"
 #include "global_user_if.h"
-#include "afk_list.h"
 #include "multi_public_chat.h"
 #include "gvar.h"
 #include "hub_cmd.h"
 #include "toolkit.h"
 #include "md5.h"
 #include "md_db.h"
+#include "users_xml.h"
+#include "db_xml.h"
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
@@ -59,12 +67,10 @@ static int client_can_accept_command(LOCAL_USER_CNX_ENTRY *luce)
 	{
 		int no_my_info;
 
-		G_LOCK(conf_file);
-		if(!e_db_int_get(conf_file,"NO_MY_INFO",&no_my_info))
+		if (!db_int_get("NO_MY_INFO", &no_my_info))
 			no_my_info=0;
-		G_UNLOCK(conf_file);
-		if(!no_my_info)
-		{
+
+		if(!no_my_info) {
 			XFIO_SET_CLOSE_STATUS(luce->user_xfio,CNX_WILL_DIE);
 			shutdown(luce->user_xfio->sock_fd,SHUT_RD);
 
@@ -224,7 +230,7 @@ static int ucmd_get_info_process(const char *cmd, LOCAL_USER_CNX_ENTRY *luce, GS
 /************************/
 static int ucmd_allow_small_event(LOCAL_USER_CNX_ENTRY *ptr, GString *str)
 {
-	if ((ptr->privilege & BOT_PRIV) == 0)
+	if (ptr->level < get_right_level(OPERATOR)) /* OPERATOR can flood */
 	{
 		ptr->msg_count++;
 		if (str)
@@ -288,7 +294,7 @@ static int ucmd_global_chat_process(const char *cmd, LOCAL_USER_CNX_ENTRY *luce,
 
 	if (ucmd_allow_small_event(luce, str) == 0)
 		goto global_chat_process_fin;
-	if ((gl_moderate_mode) && (luce->privilege == 0))
+	if ((gl_moderate_mode) && (luce->level < get_right_level(OPERATOR)))
 	{
 		send_const_str_to_luce(luce,"<Kernel> the central chat is in moderate mode|");
 		goto global_chat_process_fin;
@@ -301,14 +307,14 @@ static int ucmd_global_chat_process(const char *cmd, LOCAL_USER_CNX_ENTRY *luce,
 
 	/* be a bot or have your nick between <> */
 	if(  ((!strncmp(str->str+1, luce->user_nick->str, luce->user_nick->len)) && (!strncmp(&(str->str[1+luce->user_nick->len]),"> ",2)))
-		 || (luce->privilege & BOT_PRIV)
+		 || (luce->IsBot)
 	  )
 	{
 		gchar *str_dup;
 
 		str_dup=g_strdup(str->str);
 
-		if (luce->privilege & BOT_PRIV)
+		if (luce->IsBot)
 		{
 			char *botmsg;
 			char *t;
@@ -372,7 +378,7 @@ static int ucmd_getnicklist_process(const char *cmd, LOCAL_USER_CNX_ENTRY *luce,
 	oplst=g_string_new("$OpList ");
 
 	/* only operator and master can see the hub-security "fake" user */
-	if(luce->privilege & OP_PRIV)
+	if(luce->level >= get_right_level(OPERATOR))
 	{
 		ulst=g_string_append(ulst,"Hub-Security$$");
 		oplst=g_string_append(oplst,"Hub-Security$$");
@@ -427,10 +433,8 @@ static int ucmd_kick_process(const char *cmd, LOCAL_USER_CNX_ENTRY *luce, GStrin
 		return (0);
 	}
 
-	G_LOCK(conf_file);
-	if(!e_db_int_get(conf_file,"SILENT_KICK",&silent_kick))
+	if (!db_int_get("SILENT_KICK", &silent_kick))
 		silent_kick=0;
-	G_UNLOCK(conf_file);
 
 	t=str->str+strlen("$Kick ");
 	SKIP_SPACE(t);
@@ -444,17 +448,9 @@ static int ucmd_kick_process(const char *cmd, LOCAL_USER_CNX_ENTRY *luce, GStrin
 		/* translate the command into hub command */
 		query=g_string_new("");
 		if(silent_kick==0)
-		{
-			g_string_sprintf(query,"<%s> -tbanip \"%s\"|",luce->user_nick->str,wanted_nick);
-			process_hub_command(luce,query->str);
 			g_string_sprintf(query,"<%s> -kick \"%s\" ...|",luce->user_nick->str,wanted_nick);
-		}
 		else
-		{
-			g_string_sprintf(query,"<%s> -tbanip \"%s\"|",luce->user_nick->str,wanted_nick);
-			process_hub_command(luce,query->str);
 			g_string_sprintf(query,"<%s> -skick \"%s\" ...|",luce->user_nick->str,wanted_nick);
-		}
 		
 		process_hub_command(luce,query->str);
 		g_string_free(query,TRUE);
@@ -664,7 +660,6 @@ static int ucmd_myinfo_process(const char *cmd, LOCAL_USER_CNX_ENTRY *luce, GStr
 			have_changed=1;
 		}
 
-
 #ifdef GLIB_INT_64_BUG
 		str_to_llunsigned(fnd_size, &(new_share_size));
 #else
@@ -688,13 +683,13 @@ static int ucmd_myinfo_process(const char *cmd, LOCAL_USER_CNX_ENTRY *luce, GStr
 			have_changed=1;
 		}
 
-		if(luce->user_flag!=(((unsigned int)fnd_flag)&255))
+		if(luce->user_flag!=(((guint16)fnd_flag)&255))
 		{
-			luce->user_flag=((unsigned int)fnd_flag)&255;
+			luce->user_flag=((guint16)fnd_flag)&255;
 			have_changed=1;
 		}
 	}
-	
+
 	if(have_changed)
 	{
 		luce->ext_display_flag|=HAVE_SENT_MYINFO;
@@ -702,7 +697,7 @@ static int ucmd_myinfo_process(const char *cmd, LOCAL_USER_CNX_ENTRY *luce, GStr
 		/* send the myinfo to everyone */
 		glus_do_my_info( luce->user_nick->str, luce->user_description->str, luce->user_cnx_type,
 		                 luce->user_flag, luce->user_mail->str, luce->shared_size,
-		                 luce->ext_flag, luce->privilege, luce->client_version->str);
+		                 luce->ext_flag, luce->level, luce->client_version->str);
 
 		{
 			GString *nw;
@@ -1060,7 +1055,7 @@ static int ucmd_search_process(const char *cmd, LOCAL_USER_CNX_ENTRY *luce, GStr
 	}
 
 	luce->sr_count++;
-	if (((gl_cur_time - luce->last_hcall) < gl_sr_interval) && ((luce->privilege & BOT_PRIV)==0))
+	if (((gl_cur_time - luce->last_hcall) < gl_sr_interval) && ((luce->level >= get_right_level(OPERATOR))))
 	{
 		send_const_str_to_luce(luce,"<Kernel> search dropped. Wait 10sec between 2 searchs|");
 		g_string_free(str,TRUE);
@@ -1148,20 +1143,20 @@ static int ucmd_search_process(const char *cmd, LOCAL_USER_CNX_ENTRY *luce, GStr
 		if(sscanf(tmp_ret_adr,"%u.%u.%u.%u",&v1,&v2,&v3,&v4)!=4)
 			goto leave_on_malformed_ret;			/* the return address does not look like an IP */
 		level++;
-		if((v1>255)||(v2>255)||(v3>255)||(v4>=255))
+		if((v1>255)||(v2>255)||(v3>255)||(v4>255))
 			goto leave_on_malformed_ret;			/* malformed values */
 		level++;
 		hip=(v1<<24)|(v2<<16)|(v3<<8)|v4;
 		if(hip!=ntohl(luce->user_xfio->user_ip.s_addr))
 		{
 			static const char *err_msg[]={"Nickname of the connection and of the query are different",
-													"Port not in the range 1-65535",
-													"Not an IP",
-													"Not a valid IP",
-													"IP of the connection and of the query are different"};
+										  "Port not in the range 1-65535",
+										  "Not an IP",
+										  "Not a valid IP",
+										  "IP of the connection and of the query are different"};
 			leave_on_malformed_ret:
 
-			if((luce->privilege & HAVE_OP_KEY)==0)
+			if ((luce->level < get_right_level(OPERATOR)))
 			{
 				/* close the connection of this user */
 				XFIO_SET_CLOSE_STATUS(luce->user_xfio,CNX_WILL_DIE);
@@ -1212,7 +1207,7 @@ static int ucmd_xsearch_process(const char *cmd, LOCAL_USER_CNX_ENTRY *luce, GSt
 	}
 
 	luce->sr_count++;
-	if (((gl_cur_time - luce->last_hcall) < gl_sr_interval) && ((luce->privilege & BOT_PRIV)==0))
+	if (((gl_cur_time - luce->last_hcall) < gl_sr_interval) && ((luce->level < get_right_level(OPERATOR))))
 	{
 		send_const_str_to_luce(luce,"<Kernel> search dropped. Wait 10sec between 2 searchs|");
 		g_string_free(str,TRUE);
@@ -1338,7 +1333,7 @@ static int ucmd_md4get0_process(const char *cmd, LOCAL_USER_CNX_ENTRY *luce, GSt
 	}
 
 	luce->sr_count++;
-	if (((gl_cur_time - luce->last_hcall) < gl_sr_interval) && ((luce->privilege & BOT_PRIV)==0))
+	if (((gl_cur_time - luce->last_hcall) < gl_sr_interval) && ((luce->level < get_right_level(OPERATOR))))
 	{
 		send_const_str_to_luce(luce,"<Kernel> search dropped. Wait 10sec between 2 searchs|");
 		g_string_free(str,TRUE);
@@ -1440,7 +1435,7 @@ static int ucmd_md4set_process(const char *cmd, LOCAL_USER_CNX_ENTRY *luce, GStr
 	}
 
 	luce->sr_count++;
-	if (((gl_cur_time - luce->last_hcall) < gl_sr_interval) && ((luce->privilege & BOT_PRIV)==0))
+	if (((gl_cur_time - luce->last_hcall) < gl_sr_interval) && ((luce->level < get_right_level(OPERATOR))))
 	{
 		send_const_str_to_luce(luce,"<Kernel> search dropped. Wait 10sec between 2 searchs|");
 		g_string_free(str,TRUE);
@@ -1564,7 +1559,7 @@ int ucmd_private_chat_process(const char *cmd, LOCAL_USER_CNX_ENTRY *luce, GStri
 
 	org_cmd=g_string_new(str->str);
 	
-	if (luce->privilege == 0)
+	if (luce->level < get_right_level(OPERATOR))
 	{
 		if (ucmd_allow_small_event(luce, NULL) == 0)
 			goto leave;
@@ -1594,7 +1589,7 @@ int ucmd_private_chat_process(const char *cmd, LOCAL_USER_CNX_ENTRY *luce, GStri
 		goto leave;
 
 	t++;
-	if(strcmp(source_nick,luce->user_nick->str) && ((luce->privilege & BOT_PRIV) == 0))
+	if(strcmp(source_nick,luce->user_nick->str))
 		goto leave;			/* the source nickname does not match with connexion nickname */
 	
 #if 1
@@ -1657,8 +1652,6 @@ int ucmd_private_chat_process(const char *cmd, LOCAL_USER_CNX_ENTRY *luce, GStri
 					GLUS_SEND_TO_A_NICK(dest_nick,g_string_new(org_cmd->str));	/* don't free org_cmd */
 					org_cmd=NULL;
 				}
-				find_afk_by_nickname(dest_nick,luce->user_nick->str);
-
 				glus_free_user_info(ent);
 			}
 		}
@@ -1807,8 +1800,7 @@ int ucmd_version_process(const char *cmd, LOCAL_USER_CNX_ENTRY *luce, GString *s
 /****************************/
 int ucmd_force_send_process(const char *cmd, LOCAL_USER_CNX_ENTRY *luce, GString *str, char *xtra_param)
 {
-	if((luce->privilege & BOT_PRIV) == 0 )
-	{
+	if(!luce->IsBot) {
 		g_string_free(str,TRUE);
 		return 1;
 	}
@@ -1829,8 +1821,7 @@ int ucmd_force_send_to_process(const char *cmd, LOCAL_USER_CNX_ENTRY *luce, GStr
 	char *t;
 	char *nickname;
 
-	if((luce->privilege & BOT_PRIV) == 0 )
-	{
+	if(!luce->IsBot) {
 		g_string_free(str,TRUE);
 		return 1;
 	}
