@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 /*
-$Id: timed_out_string.c,v 2.1 2003/03/07 16:07:54 ericprev Exp $
+$Id: timed_out_string.c,v 2.6 2003/11/05 18:03:50 ericprev Exp $
 */
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -258,11 +258,7 @@ static void gui_send(const unsigned long key, const void *ptr1, int len1, const
 {
 	char	dur = '?';
 	
-	if (key == TBAN_TOSKEY)
-		dur = 't';
-	else if (key == PBAN_TOSKEY)
-		dur = 'p';
-	else if (key == CNX_TOSKEY)
+	if (key == CNX_TOSKEY)
 		dur = 'c';
 
 	if ((ptr1!=NULL) && (len1==sizeof(struct in_addr))) /* sould alway be true */
@@ -335,8 +331,6 @@ void add_tos_entry(const unsigned long key, const time_t duration, const char *p
 
 	tos_array=g_array_append_val(tos_array,nw);
 
-	if (gl_gui_mode)
-		gui_send(key, ptr1, len1, ptr2, len2, 'b');
 	G_UNLOCK(tos_array);
 }
 
@@ -348,11 +342,6 @@ static inline void delete_tos_entry(int num)
 	TOS_ENTRY *te;
 
 	te=&(g_array_index(tos_array,TOS_ENTRY,num));
-	if (gl_gui_mode)		
-		gui_send(te->key, (te->ba[0]==NULL)?NULL:te->ba[0]->data,
-						  (te->ba[0]==NULL)?0:te->ba[0]->len,
-						  (te->ba[1]==NULL)?NULL:te->ba[1]->data,
-						  (te->ba[1]==NULL)?0:te->ba[1]->len,'u');
 	if(te->ba[0])
 		g_byte_array_free(te->ba[0],TRUE);
 	if(te->ba[1])
@@ -373,9 +362,13 @@ static inline void delete_tos_entry(int num)
 /************************************************************/
 /* NOTE: this function MUST be called when the tos_array */
 /* is locked.                                            */
-/*********************************************************/
-static void delete_same_key(const unsigned long key, const char *ptr, const int len, int side)
+/*************************************************************/
+/* output: FALSE: key not found. TRUE: key found and deleted */
+/*************************************************************/
+static gboolean delete_same_key(const unsigned long key, const char *ptr, const int len, int side)
 {
+	gboolean ret=FALSE;
+
 	if(tos_array!=NULL)
 	{
 		int i;
@@ -397,9 +390,11 @@ static void delete_same_key(const unsigned long key, const char *ptr, const int
 				continue;
 				
 			delete_tos_entry(i);
+			ret=TRUE;
 			break;
 		}
 	}
+	return ret;
 }
 
 /******************************************************************/
@@ -432,10 +427,7 @@ void delete_all_same_key(const unsigned long key, GString **dest_base, const cha
 
       if((f>0) && (dest_base!=NULL))
       {
-         if(key==TBAN_TOSKEY)
-            g_string_sprintfa((*dest_base)," %i entries removed from tempbanlist\r\n",f);
-         else
-            g_string_sprintfa((*dest_base)," %i entries removed from permbanlist\r\n",f);
+			g_string_sprintfa((*dest_base)," %i entries removed from ban list\r\n",f);
       }
    }
    G_UNLOCK(tos_array);
@@ -487,9 +479,6 @@ void add_tos_entry_v1_uniq(const unsigned long key, const time_t duration, const
 		delete_same_key(key,ptr1,len1,0);
 	}
 
-	if (gl_gui_mode)
-		gui_send(key, ptr1, len1, ptr2, len2, 'b');
-
 	tos_array=g_array_append_val(tos_array,nw);
 	G_UNLOCK(tos_array);
 }
@@ -568,12 +557,16 @@ int get_tos_entry(const unsigned long key, const char *ptr, const int len, int s
 /* each entry of the tos is a triplet (key,ptr1,ptr2)       */
 /* if side==0, this function returns in *out_ptr the first x*/
 /* matching (key,ptr,x). If side==1, it is for (key,x,ptr)  */
-/************************************************************/
-void delete_this_tos_entry(const unsigned long key, const char *ptr, const int len, int side)
+/*************************************************************/
+/* output: FALSE: key not found. TRUE: key found and deleted */
+/*************************************************************/
+gboolean delete_this_tos_entry(const unsigned long key, const char *ptr, const int len, int side)
 {
+	gboolean ret;
 	G_LOCK(tos_array);
-	delete_same_key(key, ptr, len, side);
+	ret=delete_same_key(key, ptr, len, side);
 	G_UNLOCK(tos_array);
+	return ret;
 }
 
 /******************************************/
@@ -641,6 +634,26 @@ void scan_all_tos_entry(const unsigned long key, int (*fnc)(const time_t timeout
 	G_UNLOCK(tos_array);
 }
 
+/********************************************************/
+/* Return the number of entry of a key in the tos array */
+/***************************************************************/
+int number_of_tos_entry(const unsigned long key) {
+	int result=0;
+	G_LOCK(tos_array);
+	if(tos_array != NULL) {
+		int i;
+		for (i=0; i<tos_array->len; i++) {
+			TOS_ENTRY *te;
+			te=&(g_array_index(tos_array,TOS_ENTRY,i));
+			if(te->key==key) {
+				result++;
+			}
+		}
+	}
+	G_UNLOCK(tos_array);
+	return result;
+}
+
 #ifdef WIN32
 #define BUG_COMPILE 1
 #endif
@@ -673,4 +686,3 @@ void	gui_ban_list(const unsigned long key)
 	scan_all_tos_entry(key, list, NULL);
 }
 #endif
-
