@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 /*
-$Id: user_cnx_lst.c,v 2.41 2003/05/31 16:31:28 ericprev Exp $
+$Id: user_cnx_lst.c,v 2.55 2003/11/22 13:47:08 blusseau Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -35,7 +35,14 @@ $Id: user_cnx_lst.c,v 2.41 2003/05/31 16:31:28 ericprev Exp $
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#ifdef WITH_GCRYPT
+#include <gcrypt.h>
+#define MD4_DIGEST_LENGTH 16
+#define MD4_ALGO(source,len,dest)   gcry_md_hash_buffer(GCRY_MD_MD4,dest,source,len)
+#else
 #include <openssl/md4.h>
+#define MD4_ALGO MD4
+#endif
 #include <glib.h>
 
 #include "gvar.h"
@@ -48,6 +55,8 @@ $Id: user_cnx_lst.c,v 2.41 2003/05/31 16:31:28 ericprev Exp $
 #include "timed_out_string.h"
 #include "tos_key.h"
 #include "md5.h"
+#include "users_xml.h"
+#include "db_xml.h"
 
 /* hash table of connection */
 /* the key is the nickname (stored inside the value) */
@@ -73,8 +82,10 @@ HL_MUTEX user_cnx_entry_lock = HL_MUTEX_INIT;
 /******************************************************************************************/
 /* output: address of the newly created and inserted LOCAL_USER_CNX_ENTRY * */
 /****************************************************************************/
-LOCAL_USER_CNX_ENTRY *uc_create_entry(XF_IO *xfio, char *nickname, guint64 shared_size, char *umail, char *udesc, char *ucliver,
-							unsigned int user_flag, unsigned int ext_flag, int privilege_mask)
+LOCAL_USER_CNX_ENTRY *uc_create_entry(XF_IO *xfio, char *nickname, guint64 shared_size,
+									  char *umail, char *udesc, char *ucliver,
+									  guint16 user_flag, guint16 ext_flag,
+									  gint16 level, gboolean isbot)
 {
 	LOCAL_USER_CNX_ENTRY *luce;
 
@@ -99,16 +110,16 @@ LOCAL_USER_CNX_ENTRY *uc_create_entry(XF_IO *xfio, char *nickname, guint64 share
 
 	luce->user_flag=user_flag|1;		/* the lowest bit is always set */
 	luce->ext_flag=ext_flag;
-	luce->privilege=privilege_mask;
-	if(ucliver!=NULL)
+	if (ucliver!=NULL)
 		luce->client_version=g_string_new(ucliver);
 	else
 		luce->client_version=g_string_new("");
 
-	if(luce->privilege)	/* op users do not need timeout sytem */
-		luce->last_time=gl_cur_time + 5 * 356 * 36000;
-	else
-		luce->last_time=gl_cur_time;
+	luce->last_time=gl_cur_time;
+
+	luce->level=level;
+	luce->IsBot=isbot;
+	
 	luce->last_hcall=gl_cur_time;
 	luce->last_msg=0;
 	luce->msg_count=0;
@@ -261,15 +272,10 @@ static gboolean uc_close_long_time_idle(gpointer key, gpointer value, gpointer u
 	int max_idle=GPOINTER_TO_INT(user_data);
 	LOCAL_USER_CNX_ENTRY *luce=value;
 
-	if(luce->privilege==0)
-	{
-		/* simple users need system timeout, other don't */
-		if ((gl_cur_time - luce->last_time) > max_idle)
-		{
-			printf("user %s leaves: time out.\n", luce->user_nick->str);
-			end_user_connection(luce,FALSE);		/* don't delete luce ptr, it is handle after */
-			return TRUE;		/* luce was destroyed -> delete key/value pair */
-		}
+	if ((gl_cur_time - luce->last_time) > max_idle) {
+		printf("user %s leaves: time out.\n", luce->user_nick->str);
+		end_user_connection(luce,FALSE);		/* don't delete luce ptr, it is handle after */
+		return TRUE;		/* luce was destroyed -> delete key/value pair */
 	}
 
 	return FALSE;		/* keep key/value pair */
@@ -391,7 +397,7 @@ static GLUS_USER_INFO *uc_glus_get_user_info(const char *nickname)
 		gui->user_mail=g_string_new("");
 	gui->user_flag=luce->user_flag;
 	gui->ext_flag=luce->ext_flag;
-	gui->privilege=luce->privilege;
+	gui->level=luce->level;
 	if(luce->client_version)
 		gui->client_version=g_string_new(luce->client_version->str);
 	else
@@ -434,7 +440,7 @@ static GLUS_USER_INFO *uc_glus_get_user_info_from_luce(LOCAL_USER_CNX_ENTRY *luc
 		gui->user_mail=g_string_new("");
 	gui->user_flag=luce->user_flag;
 	gui->ext_flag=luce->ext_flag;
-	gui->privilege=luce->privilege;
+	gui->level=luce->level;
 	if(luce->client_version)
 		gui->client_version=g_string_new(luce->client_version->str);
 	else
@@ -469,7 +475,7 @@ static void uc_get_nick_lists_one(gpointer key, gpointer value, gpointer user_da
 		g_string_append(lists[0],luce->user_nick->str);
 		g_string_append(lists[0],"$$");
 	
-		if(luce->privilege & HAVE_OP_KEY)
+		if(luce->level >= get_right_level(OPERATOR))
 		{
 			g_string_append(lists[1],luce->user_nick->str);
 			g_string_append(lists[1],"$$");
@@ -527,7 +533,6 @@ static int uc_kick_named_user(const char *kicking_nickname, const char *nickname
 {
 	LOCAL_USER_CNX_ENTRY *luce; /* the kicked user */
 	LOCAL_USER_CNX_ENTRY *kuce; /* the kicking user */
-	/* int temp_ban; */
 	int bot_kick;
 	GString *str;
 
@@ -541,29 +546,8 @@ static int uc_kick_named_user(const char *kicking_nickname, const char *nickname
 	}
 	
 	/* retrieve the temp ban duration when a kick occurs */
-	G_LOCK(conf_file);
-	/* if(!e_db_int_get(conf_file,"KICKTEMPBAN",&temp_ban)) */
-		/* temp_ban=0; */
-	if(!e_db_int_get(conf_file,"DISPLAY_BOT_KICK",&bot_kick))
+	if (!db_int_get("DISPLAY_BOT_KICK", &bot_kick))
 		bot_kick=1;
-	G_UNLOCK(conf_file);
-
-	/* don't prevent anyone from getting disconnected at all */
-	/* the disconnecting/kicking decision should be made before */
-	/* executing such a command */
-
-	/* if (luce->ext_flag & SHILD_MODE) */
-	/* { */
-		/* HL_UNLOCK_READ(user_cnx_entry_lock); */
-		/* return 0; */	/* user found, nothing to do */
-	/* } */
-
-	/* The ban was done in a separate 'do_banip'-funktion-call before */
-
-	/* if (temp_ban != 0) */
-	/* { */
-		/* add_tos_entry_v1_uniq(TBAN_TOSKEY,temp_ban*60,(void*)(&(luce->user_xfio->user_ip)),sizeof(struct in_addr),NULL,0); */
-	/* } */
 
 	XFIO_SET_CLOSE_STATUS(luce->user_xfio,CNX_WILL_DIE);	/* flush queued message but only if possible */
 	shutdown(luce->user_xfio->sock_fd,SHUT_RD);		/* we don't want to receive more data from this connexion */
@@ -579,12 +563,21 @@ static int uc_kick_named_user(const char *kicking_nickname, const char *nickname
 		g_string_sprintf(str,"<Hub-Security> The user %s was kicked. IP: %s|",
 			 nickname,inet_ntoa(luce->user_xfio->user_ip));
 	}
+
+	/* set the ban TOS entry */
+	{
+		GLUS_USER_INFO *gui;
+		gui = glus_get_user_info(nickname);
+		if (gui == NULL)
+			fprintf(stderr,"uc_ban_named_user: user exists for user_cnx_entry_get_by_nickname but not for glus_get_user_info...weird.\n");
+		else
+			glus_free_user_info(gui);
+	}
+
 	/* message only to the kicked user ??? I don't think so ;) */
-	/* send_stolen_gstr_to_luce(luce,str);	*/	/* don't free str */
 	if(kuce!=NULL)
 	{
-		if(bot_kick==0 && (kuce->privilege & BOT_PRIV))
-		{
+		if (bot_kick==0 && kuce->IsBot) {
 			g_string_free(str,TRUE);
 		}
 		else
@@ -604,6 +597,157 @@ static int uc_kick_named_user(const char *kicking_nickname, const char *nickname
 	return 0;
 }
 
+/****************************************/
+/* ban a user having the given nickname */
+/****************************************/
+/* output: 0=ok else user not found */
+/************************************/
+static int uc_ban_named_user(const char *kicking_nickname, const char *nickname, unsigned long duration)
+{
+	LOCAL_USER_CNX_ENTRY *luce; /* the kicked user */
+	LOCAL_USER_CNX_ENTRY *kuce; /* the kicking user */
+	GString *str;
+	int bot_kick;
+
+	HL_LOCK_READ(user_cnx_entry_lock);
+	luce=user_cnx_entry_get_by_nickname(nickname);
+	kuce=user_cnx_entry_get_by_nickname(kicking_nickname);
+	if(luce==NULL)
+	{
+		HL_UNLOCK_READ(user_cnx_entry_lock);
+		return 1;		/* user not found */
+	}
+
+	if (!db_int_get("DISPLAY_BOT_KICK", &bot_kick))
+		bot_kick=1;
+	
+	str=g_string_new("");
+	if(kicking_nickname!=NULL)
+	{
+		g_string_sprintf(str,"<Hub-Security> The user %s was banned by %s. IP: %s|",
+		 	nickname,kicking_nickname,inet_ntoa(luce->user_xfio->user_ip));
+	}
+	else
+	{
+		g_string_sprintf(str,"<Hub-Security> The user %s was banned. IP: %s|",
+			 nickname,inet_ntoa(luce->user_xfio->user_ip));
+	}
+
+	/* set the ban TOS entry */
+	{
+		GLUS_USER_INFO *gui;
+		gui = glus_get_user_info(nickname);
+		if(gui==NULL)
+		{
+			fprintf(stderr,"uc_ban_named_user: user exists for user_cnx_entry_get_by_nickname but not for glus_get_user_info...weird.\n");
+		}
+		else
+		{
+			add_tos_entry_v1_uniq(BAN_TOSKEY,duration,(void*)(&(gui->user_ip)),sizeof(struct in_addr),nickname,strlen(nickname));
+			glus_free_user_info(gui);
+		}
+	}
+
+	/* message only to the banned user ??? I don't think so ;) */
+	if(kuce!=NULL)
+	{
+		if (bot_kick==0 && kuce->IsBot) {
+			g_string_free(str,TRUE);
+		}
+		else
+			GLUS_SEND_TO_EVERYONE(str);      /* don't free str */
+	}
+	else
+	{
+		if(bot_kick==0)
+		{
+			g_string_free(str,TRUE);
+		}
+		else
+			GLUS_SEND_TO_EVERYONE(str);      /* don't free str */
+	}
+
+	HL_UNLOCK_READ(user_cnx_entry_lock);
+	return 0;
+}
+
+/************************************/
+/* ban a user having the given IP   */
+/************************************/
+/* output: 0=ok else user not found */
+/************************************/
+static int uc_ban_ip(const char *kicking_nickname, struct in_addr ip, unsigned long duration)
+{
+	LOCAL_USER_CNX_ENTRY *kuce; /* the kicking user */
+	GString *str;
+	int bot_kick;
+
+	HL_LOCK_READ(user_cnx_entry_lock);
+	kuce=user_cnx_entry_get_by_nickname(kicking_nickname);
+	
+	if (!db_int_get("DISPLAY_BOT_KICK", &bot_kick))
+		bot_kick=1;
+	
+	str=g_string_new("");
+	if(kicking_nickname != NULL)
+	{
+		g_string_sprintf(str,"<Hub-Security> IP %s was banned by %s.|",
+						 inet_ntoa(ip),kicking_nickname);
+	}
+	else	
+	{
+		g_string_sprintf(str,"<Hub-Security> IP %s was banned.|",
+						 inet_ntoa(ip));
+	}
+	
+	/* set the ban TOS entry */
+	add_tos_entry_v1_uniq(BAN_TOSKEY,duration, (void*)&ip, sizeof(struct in_addr), NULL, 0);
+	
+	/* message only to the banned user ??? I don't think so ;) */
+	if (kuce!=NULL)
+	{
+		if (bot_kick == 0 && kuce->IsBot) {
+			g_string_free(str,TRUE);
+		}
+		else
+			GLUS_SEND_TO_EVERYONE(str);      /* don't free str */
+	}
+	else
+	{
+		if (bot_kick == 0)
+		{
+			g_string_free(str,TRUE);
+		}
+		else
+			GLUS_SEND_TO_EVERYONE(str);      /* don't free str */
+	}
+	
+	HL_UNLOCK_READ(user_cnx_entry_lock);
+	return 0;
+}
+
+/************************************************/
+/* unban a user having the given nickname or IP */
+/************************************************/
+/* output: 0=ok else user not found */
+/************************************/
+static int uc_uban_named_user(const char *nickname)
+{
+	/* first, try to remove the value as a nickname */
+	if(delete_this_tos_entry(BAN_TOSKEY,(void*)nickname,strlen(nickname),1)==FALSE)
+	{
+		struct in_addr inp;
+
+		if(inet_aton(nickname,&inp)==0)
+			return 1;		/* the value is not an IP */
+		if(delete_this_tos_entry(BAN_TOSKEY,(void*)&inp,sizeof(inp),0)==FALSE)
+			return 1;		/* the value is an IP but it is not found */
+		return 0;	/* the value was a known IP */
+	}
+	else
+		return 0;	/* the value was a nickname */
+}
+
 /***********************************************/
 /* disconnect a user having the given nickname */
 /***********************************************/
@@ -772,7 +916,7 @@ static void uc_user_do_quit(const char *nickname)
 /******************************************************/
 /* function used to say a "$MyINFO" on the connection */
 /******************************************************/
-static void uc_user_do_my_info(const char *nickname, const char *description, const char *user_cnx_type, const unsigned int user_flag, const char *user_mail, const guint64 shared_size, const unsigned int ext_flag, const unsigned int privilege, const char *client_version)
+static void uc_user_do_my_info(const char *nickname, const char *description, const char *user_cnx_type, const guint16 user_flag, const char *user_mail, const guint64 shared_size, const guint16 ext_flag, const gint16 level, const char *client_version)
 {
 	GString *str;
 	GString_ref *ref;
@@ -903,15 +1047,12 @@ static void uc_ged_periodic(const struct ged_callbacks *ged)
 {
 	/* check for inactivity in all cnx every 10 sec if the MAX_CNX_IDLE key exists */
 	int max_idle;
-	int have_idle_value;
+	gboolean have_idle_value;
 
-	G_LOCK(conf_file);
-	have_idle_value=e_db_int_get(conf_file,"MAX_CNX_IDLE",&max_idle);
-	G_UNLOCK(conf_file);
+	have_idle_value=db_int_get("MAX_CNX_IDLE", &max_idle);
 
-	if(have_idle_value)
-	{
-		if(max_idle<10)
+	if (have_idle_value) {
+		if (max_idle < 10)
 			max_idle=10;
 
 		HL_LOCK_READ(user_cnx_entry_lock);
@@ -953,6 +1094,9 @@ static GLOBAL_USER_CALLBACKS user_cnx_entry_glus=
 						uc_glus_get_nick_lists,
 						uc_add_hub_users_stat,
 						uc_kick_named_user,
+						uc_ban_named_user,
+						uc_ban_ip,
+						uc_uban_named_user,
 						uc_disconnect_named_user,
 						uc_user_is_disconnected,
 						uc_user_do_hello,
