@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 /*
-$Id: xs_init.c,v 2.10 2003/05/29 14:01:50 ericprev Exp $
+$Id: xs_init.c,v 2.16 2003/11/02 21:34:12 blusseau Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -48,10 +48,11 @@ $Id: xs_init.c,v 2.10 2003/05/29 14:01:50 ericprev Exp $
 #include <glib.h>
 #include "gvar.h"
 #include "xs_init.h"
-#include "passwd.h"
+#include "users_xml.h"
 #include "global_user_if.h"
 #include "user_cnx_lst.h"
 #include "hub_cmd.h"
+#include "db_xml.h"
 
 #ifdef sun
   EXTERN_C void boot_DynaLoader (CV* cv);
@@ -292,11 +293,9 @@ XS(perl_redirect)
 		XSRETURN_IV(1);
 	}
 	
-	G_LOCK(conf_file);
-	rd=e_db_str_get(conf_file,"REDIR_ADDR");
-	G_UNLOCK(conf_file);
-	if((rd!=NULL)&&(strlen(rd)!=0))
-	{
+	rd=db_str_get("REDIR_ADDR");
+
+	if ((rd != NULL) && (strlen(rd) != 0)) {
 		rd_msg=g_string_new("");
 		/* first the message to display */
 		g_string_sprintf(rd_msg,"<Hub-Security> You are being redirected to %s.|",rd);
@@ -419,29 +418,46 @@ XS(perl_nick_duration)
 	XSRETURN_IV((int)duration);
 }
 
-/**************************************/
-/* dchub::nick_type( string nickname) */
-/**************************************/
-XS(perl_nick_type)
+/***************************************/
+/* dchub::nick_level( string nickname) */
+/***************************************/
+XS(perl_nick_level)
 {
-	char	type[2];
+	gint16 result;
 	dXSARGS;
 	if(items != 1)		/* number of items in the stack */
 	{
-		printf("dchub::nick_type : wrong number of parameters (1).\n");
+		printf("dchub::nick_level : wrong number of parameters (1).\n");
 		XSRETURN_UNDEF;
 	}
 
 	if(!SvPOK(ST(0)))
 	{
-		printf("dchub::nick_type : invalid parameters (string required).\n");
+		printf("dchub::nick_level : invalid parameters (string required).\n");
 		XSRETURN_UNDEF;
 	}
 
-	type[1] = '\0';
-	if (get_account_type(SvPVX(ST(0)), type) == 0)
-		XSRETURN_PV("U");		/* unregistered user */
-	XSRETURN_PV(type);
+	if (!xml_get_user(SvPVX(ST(0)), &result, NULL, NULL, NULL))
+		result=0; /* if user is not found return level 0 */
+	
+	XSRETURN_IV(result);
+}
+
+/*************************************/
+/* dchub::right_level( string right) */
+/*************************************/
+XS(perl_right_level) {
+	dXSARGS;
+	if(items != 1) {		/* number of items in the stack */
+		printf("dchub::right_level : wrong number of parameters (1).\n");
+		XSRETURN_UNDEF;
+	}
+	if(!SvPOK(ST(0)))
+	{
+		printf("dchub::right_level : invalid parameters (string required).\n");
+		XSRETURN_UNDEF;
+	}
+	XSRETURN_IV(get_right_level(SvPVX(ST(0))));
 }
 
 /**************************/
@@ -577,10 +593,8 @@ XS(perl_get_max_users)
 	}
 	else
 	{
-		G_LOCK(conf_file);
-		if(!e_db_int_get(conf_file,"MAXUSER",&max_nb))
+		if(!db_int_get("MAXUSER", &max_nb))
 			max_nb=100;
-		G_UNLOCK(conf_file);
 	}
 
 	XSRETURN_IV(max_nb);
@@ -600,9 +614,7 @@ XS(perl_get_txtdescription)
 		XSRETURN_UNDEF;
 	}
 
-	G_LOCK(conf_file);
-	hubdesc=e_db_str_get(conf_file,"HUBDESC");
-	G_UNLOCK(conf_file);
+	hubdesc=db_str_get("HUBDESC");
 	
 	/* same as XSRETURN_PV(hubdesc) but with g_free inside */
 	XST_mPV(0,hubdesc);
@@ -624,9 +636,7 @@ XS(perl_get_hubip)
 		XSRETURN_UNDEF;
 	}
 
-	G_LOCK(conf_file);
-	hubdesc=e_db_str_get(conf_file,"HUBADDR");
-	G_UNLOCK(conf_file);
+	hubdesc=db_str_get("HUBADDR");
 	
 	/* same as XSRETURN_PV(hubdesc) but with g_free inside */
 	XST_mPV(0,hubdesc);
@@ -639,7 +649,12 @@ XS(perl_get_hubip)
 /**********************************/
 XS(perl_db_get)
 {
-	char *str;
+	char *str_value;
+	int int_value;
+	float float_value;
+	int type;
+	gboolean result;
+	
 	dXSARGS;
 	if(items != 1)		/* number of items in the stack */
 	{
@@ -653,18 +668,31 @@ XS(perl_db_get)
 		XSRETURN_UNDEF;
 	}
 
-	G_LOCK(conf_file);
-	str=e_db_str_get(conf_file,SvPVX(ST(0)));
-	G_UNLOCK(conf_file);
-	if(str==NULL)
-	{
-		XSRETURN_UNDEF;
-	}
-
-	/* same as XSRETURN_PV(str) but with free inside */
-	XST_mPV(0,str);
-	free(str);
-	XSRETURN(1);
+	type=db_type_get(SvPVX(ST(0)));
+	switch (type) {
+		case TYPE_STR:
+			str_value=db_str_get(SvPVX(ST(0)));
+			if (str_value != NULL) {
+				/* same as XSRETURN_PV(str) but with free inside */
+				XST_mPV(0,str_value);
+				free(str_value);
+				XSRETURN(1);
+			}
+			break;
+			
+		case TYPE_INT:
+			result=db_int_get(SvPVX(ST(0)),&int_value);
+			if (result == TRUE)
+				XSRETURN_IV(int_value);	
+			break;
+			
+		case TYPE_FLOAT:
+			result=db_float_get(SvPVX(ST(0)),&float_value);
+			if (result == TRUE)
+				XSRETURN_NV(float_value);	
+			break;
+	}			
+	XSRETURN_UNDEF;	
 }
 
 /*************************************************/
@@ -685,9 +713,7 @@ XS(perl_db_set)
 		XSRETURN_UNDEF;
 	}
 
-	G_LOCK(conf_file);
-	e_db_str_set(conf_file,SvPVX(ST(0)),SvPV_nolen(ST(1)));
-	G_UNLOCK(conf_file);
+	db_str_set(SvPVX(ST(0)),SvPV_nolen(ST(1)));
 
 	XSRETURN_IV(1);
 }
@@ -761,7 +787,11 @@ void xs_init(pTHXo)
 	newXS("dchub::oplist",perl_oplist,"xs_init.c");
 	newXS("dchub::nickinfo",perl_nickinfo,"xs_init.c");
 	newXS("dchub::nick_duration",perl_nick_duration,"xs_init.c");
-	newXS("dchub::nick_type",perl_nick_type,"xs_init.c");
+	newXS("dchub::nick_level",perl_nick_level,"xs_init.c");
+	newXS("dchub::right_level",perl_right_level,"xs_init.c");
+
+	/* Obsolete since 0.5.0 will be remove (same function as dchub::nick_level) */
+	newXS("dchub::nick_type",perl_nick_level,"xs_init.c");
 
 	newXS("dchub::nick_ip", perl_nick_ip, "xs_init.c");	/* Morra: get users ip */
 	newXS("dchub::get_max_users",perl_get_max_users,"xs_init.c");
